/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.ImportDefinition");

dojo.require("rmps.core.resource.Resource");

var PATH_MAP = "http://jazz.net/xmlns/rmps/importer/1.0/pathMap";
var PATH_MAP_VALUE = "http://jazz.net/xmlns/rmps/importer/1.0/pathMapValue";
var PATH_MAP_VAR = "http://jazz.net/xmlns/rmps/importer/1.0/pathMapVar";

var ECLIPSE_WORKSPACE_PATH = "http://jazz.net/xmlns/rmps/importer/1.0/eclipseWorkspacePath";

var FILE_WORKSPACE = "http://jazz.net/xmlns/rmps/importer/1.0/fileWorkspace";
var ECLIPSE_PROJECT = "http://jazz.net/xmlns/rmps/importer/1.0/eclipseProject";
var ECLIPSE_PROJECT_DIR_NAME = "http://jazz.net/xmlns/rmps/importer/1.0/directoryName";
var ECLIPSE_PROJECT_ALIAS = "http://jazz.net/xmlns/rmps/importer/1.0/projectAlias";
var FILE_WORKSPACE_PATH = "http://jazz.net/xmlns/rmps/importer/1.0/fileWorkspacePath";

var ENGINE_ID = "http://jazz.net/xmlns/rmps/importer/1.0/engineId";
var SAMPLE_MODEL_IMPORT = "http://jazz.net/xmlns/rmps/importer/1.0/sample";
var ANT_SCRIPT_PATH = "http://jazz.net/xmlns/rmps/importer/1.0/antScriptPath";

var SCHEDULE = "http://jazz.net/xmlns/rmps/importer/1.0/schedule";
var SCHEDULE_INTERVAL = "http://jazz.net/xmlns/rmps/importer/1.0/scheduleInterval";
var SCHEDULE_TIME = "http://jazz.net/xmlns/rmps/importer/1.0/scheduleTime";
var SCHEDULE_WEEK_DAY = "http://jazz.net/xmlns/rmps/importer/1.0/scheduleWeekDay";
var SCHEDULE_BASELINE_NAME = "http://jazz.net/xmlns/rmps/importer/1.0/baselineName";
var SCHEDULE_BASELINE = "http://jazz.net/xmlns/rmps/importer/1.0/createBaseline";
var CREATED_VARIABLE = "${created}";

var TRANSFER_TIME_OUT = "http://jazz.net/xmlns/rmps/importer/1.0/transferTimeOut";
var ANT_EXECUTION_TIME_OUT = "http://jazz.net/xmlns/rmps/importer/1.0/antExecutionTimeOut";

rmps.importer.ImportDefinitionSchedule = {
	interval: SCHEDULE_INTERVAL,
	time: SCHEDULE_TIME
};

dojo.declare("rmps.importer.ImportDefinition", rmps.core.resource.Resource, {
	
	engineId: null,
	antScriptPath: null,
	
	eclipseWorkspacePath: null,
	sampleModelImport: null, // "true" or "false"
	
	workspaces: null, // [{path: "", projects: []}], projects = [{alias: "", directoryName: ""}]
	
	pathMaps: null, // [{variable: "", value: ""}]
	
	contentType: "application/rdf+json",
	
	schedule: null, // ImportDefinitionSchedule|null=disabled
	scheduleInterval: null, // number of minutes
	scheduleTime: null, // format xx:xx indicating a time in 24h format up to minutes - GMT
	scheduleWeekDays: null, // [] - weekday is the number from 1 to 7 matching "monday" to "sunday"
	scheduleBaseline: false, //true/false - whether or not to create a baseline after scheduled import,
	scheduleBaselineName: CREATED_VARIABLE, //name of the baseline, if creating
	
	transferTimeOut: null,
	antExecutionTimeOut: null,	
	
	constructor: function() {
		this.workspaces = [];
		this.pathMaps = [];
		this.scheduleWeekDays = [];
	},
	
	toRdfJson: function() {
		var nodeId = 1;
		
		function createNode() {
			return {
				type: "bnode",
				value: "_:b" + nodeId++
			};
		}
		
		function createInteger(value) {
			var result = createLiteral("" + value);
			result.datatype = "http://www.w3.org/2001/XMLSchema#int";
			return result;
		}
		
		function createBoolean(value) {
			var result = createLiteral("" + value);
			result.datatype = "http://www.w3.org/2001/XMLSchema#boolean";
			return result;
		}
		
		function createLiteral(value) {
			return {
				type: "literal",
				value: value
			};
		}
		
		function createUri(value) {
			return {
				type: "uri",
				value: value
			};
		}
		
		var obj = {};
		var pathMaps = [];
		var workspaces = [];
		var main = {};
		
		obj[this.uri] = main;
		
		// engine id
		if (this.engineId) {
			main[ENGINE_ID] = [createLiteral(this.engineId)];
		}
		
		// ant script
		if (this.antScriptPath) {
			main[ANT_SCRIPT_PATH] = [createLiteral(this.antScriptPath)];
		}
		
		// schedule
		if (this.schedule != null) {
			main[SCHEDULE] = [createUri(this.schedule)];
		}
			
		if (this.scheduleInterval != null) {
			main[SCHEDULE_INTERVAL] = [createInteger(this.scheduleInterval)];
		}
		
		if (this.scheduleTime) {
			main[SCHEDULE_TIME] = [createLiteral(this.scheduleTime.join(':'))];
		}
		
		if (this.scheduleWeekDays.length > 0) {
			main[SCHEDULE_WEEK_DAY] = [];
			for (var i = 0; i < this.scheduleWeekDays.length; ++i) {
				main[SCHEDULE_WEEK_DAY].push(createInteger(this.scheduleWeekDays[i]));
			}
		}
		
		if (this.scheduleBaselineName) {
			main[SCHEDULE_BASELINE_NAME] = [createLiteral(this.scheduleBaselineName)];
		}
		
		main[SCHEDULE_BASELINE] = [createBoolean(this.scheduleBaseline)];
		
		if (this.eclipseWorkspacePath) {
			main[ECLIPSE_WORKSPACE_PATH] = [createLiteral(this.eclipseWorkspacePath)];
		}
		
		if (this.sampleModelImport) {
			main[SAMPLE_MODEL_IMPORT] = [createBoolean(this.sampleModelImport)];
		}
		
		// pathmaps
		main[PATH_MAP] = pathMaps;
		for (var i = 0; i < this.pathMaps.length; ++i) {
			var pathMap = this.pathMaps[i];
			var pathMapNode = createNode();
			pathMaps.push(pathMapNode);
			
			var value = {};
			value[PATH_MAP_VALUE] = [createLiteral(pathMap.value)];
			value[PATH_MAP_VAR] = [createLiteral(pathMap.variable)];
			obj[pathMapNode.value] = value;
		}
		
		// workspaces
		main[FILE_WORKSPACE] = workspaces;
		for (var i = 0; i < this.workspaces.length; ++i) {
			var workspace = this.workspaces[i];
			var workspaceNode = createNode();
			workspaces.push(workspaceNode);
			
			var projects = [];
			var value = {};
			value[FILE_WORKSPACE_PATH] = [createLiteral(workspace.path)];
			value[ECLIPSE_PROJECT] = projects;
			obj[workspaceNode.value] = value;
			
			for (var j = 0; j < workspace.projects.length; ++j) {
				// add blank node _:b4 to projects list
				var eclipseProjectNode = createNode();
				projects.push(eclipseProjectNode);
				
				// add triple _:b4 rmps/directoryName "..."
				var project = {};
				project[ECLIPSE_PROJECT_ALIAS] = project[ECLIPSE_PROJECT_DIR_NAME] = [createLiteral(workspace.projects[j].directoryName)];
				if (workspace.projects[j].alias.length > 0) {
					project[ECLIPSE_PROJECT_ALIAS] = [createLiteral(workspace.projects[j].alias)];
				}
				
				obj[eclipseProjectNode.value] = project;
			}
		}
		
		if (this.transferTimeOut != null) {
			main[TRANSFER_TIME_OUT] = [createInteger(this.transferTimeOut)];
		}
		
		if (this.antExecutionTimeOut != null) {
			main[ANT_EXECUTION_TIME_OUT] = [createInteger(this.antExecutionTimeOut)];
		}
		
		return dojo.toJson(obj);
	},
	
	parseRdfJson: function(rdf, ioArgs) {
		function pathMapComparator(a, b) {
			return a.variable < b.variable ? 1 : a.variable == b.variable ? 0 : -1;
		}
		
		function workspaceComparator(a, b) {
			return a.path < b.path ? 1 : a.path == b.path ? 0 : -1;
		}
		
		function projectComparator(a, b) {
			return a.name < b.name ? 1 : a.name == b.name ? 0 : -1;
		}
		
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("Last-Modified")) {
				this.lastModified = new Date(xhr.getResponseHeader("Last-Modified"));
			}
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
		}
		
		for (var key in rdf) {
			if (rdf[key][ENGINE_ID] || rdf[key][SAMPLE_MODEL_IMPORT]) {
				this.uri = key;
				break;
			}
		}
		var main = rdf[this.uri];
		
		// engine id
		if (main[ENGINE_ID]) {
			this.engineId = main[ENGINE_ID][0].value;
		}
		
		if (main[SAMPLE_MODEL_IMPORT]) {
			this.sampleModelImport = main[SAMPLE_MODEL_IMPORT][0].value == "true" ? true : false;
		}
		
		// ant script
		if (main[ANT_SCRIPT_PATH]) {
			this.antScriptPath = main[ANT_SCRIPT_PATH][0].value;
		}
		
		// schedule
		if (main[SCHEDULE]) {
			this.schedule = main[SCHEDULE][0].value;
		}
			
		if (main[SCHEDULE_INTERVAL]) {
			this.scheduleInterval = parseInt(main[SCHEDULE_INTERVAL][0].value);
		}
		
		if (main[SCHEDULE_TIME]) {
			this.scheduleTime = main[SCHEDULE_TIME][0].value.split(':');
		}
		
		var weekdays = main[SCHEDULE_WEEK_DAY];
		if (weekdays) {
			for (var i = 0; i < weekdays.length; ++i) {
				this.scheduleWeekDays.push(parseInt(weekdays[i].value));
			}
			this.scheduleWeekDays.sort();
		}
		
		if (main[SCHEDULE_BASELINE_NAME]) {
			this.scheduleBaselineName = main[SCHEDULE_BASELINE_NAME][0].value;
		}
		
		if (main[SCHEDULE_BASELINE]) {
			this.scheduleBaseline = main[SCHEDULE_BASELINE][0].value === "true";
		}
		
		// eclipse workspace path
		if (main[ECLIPSE_WORKSPACE_PATH]) {
			this.eclipseWorkspacePath = main[ECLIPSE_WORKSPACE_PATH][0].value;
		}
		
		// workspaces
		var workspaces = main[FILE_WORKSPACE];
		if (workspaces) {
			for (var i = 0; i < workspaces.length; ++i) {
				var workspace = rdf[workspaces[i].value];
				
				var obj = {
					projects: [],
					path: workspace[FILE_WORKSPACE_PATH][0].value
				};
				this.workspaces.push(obj);
				
				var projects = workspace[ECLIPSE_PROJECT];
				for (var j = 0; j < projects.length; ++j) {
					var project = rdf[projects[j].value];
					obj.projects.push({
							directoryName: project[ECLIPSE_PROJECT_DIR_NAME][0].value,
							alias: project[ECLIPSE_PROJECT_ALIAS][0].value
						});
				}
				
				obj.projects.sort(projectComparator);
			}
			this.workspaces.sort(workspaceComparator);
		}
		
		// path maps
		var pathMaps = main[PATH_MAP];
		if (pathMaps) {
			for (var i = 0; i < pathMaps.length; ++i) {
				var pathMap = rdf[pathMaps[i].value];
				
				var obj = {
					variable: pathMap[PATH_MAP_VAR][0].value,
					value: pathMap[PATH_MAP_VALUE][0].value
				};
				this.pathMaps.push(obj);
			}
			
			this.pathMaps.sort(pathMapComparator);
		}
		
		if (main[TRANSFER_TIME_OUT]) {
			this.transferTimeOut = parseInt(main[TRANSFER_TIME_OUT][0].value);
		}
		
		if (main[ANT_EXECUTION_TIME_OUT]) {
			this.antExecutionTimeOut = parseInt(main[ANT_EXECUTION_TIME_OUT][0].value);
		}
	}
});
})();