/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.ImportDefinitionEditor");

dojo.require("rmps.ui.EditableCollapsibleSection");
dojo.require("rmps.ui.CollapsibleSection");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.util.array");
dojo.require("jazz.app.ua.TextHoverHelp");

dojo.require("jazz.ui.HoverPopup");

dojo.require("rmps.importer.ImportClient");

dojo.require("rmps.core.error");

dojo.require("rmps.importer.util");

dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.importer", "ImportDefinitionEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.importer", "ImportDefinitionEditorMessages");

var ProcessConstants = rmps.importer.util.process;
var ImportClient = rmps.importer.ImportClient;

var ImportDefinitionType = {
	WORKSPACE : 1,
	PROJECT : 2,
	SAMPLE : 3
}

var connectOnInput = function(obj, widget, method) {
	if (dojo.isMoz || dojo.isOpera) {
		widget.connect(obj, "oninput", method);
	} else {
		widget.connect(obj, "onkeydown", method);
		widget.connect(obj, "onkeyup", method);
		widget.connect(obj, "onpaste", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "oncut", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "onblur", method);
	}
};

var createRequiredNode = function() {
	var reqNode = dojo.create("span");
	reqNode.className = "required";
	reqNode.innerHTML = "*";
	reqNode.title = messages.requiredTitle;
	return reqNode;
};

var DIRTY_GROUP_ID = "rmps.importer.ImportDefinitionEditor.dirty";

var PAGE_UAKEY = "com.ibm.xtools.rmps.web.importer.page_ImportDefinitionEditor001";

var IMPORT_ENGINE_UAKEY = "com.ibm.xtools.rmps.web.importer.text_ImportDefinitionEditor001";
var ANT_SCRIPT_UAKEY = "com.ibm.xtools.rmps.web.importer.text_ImportDefinitionEditor002";
var ECLIPSE_WORKSPACE_PATH_UAKEY = "com.ibm.xtools.rmps.web.importer.text_ImportDefinitionEditor003";
var SNAPSHOT_NAME_UAKEY = "com.ibm.xtools.rmps.web.importer.text_ImportDefinitionEditor004";

var SCHEDULE_UAKEY = "com.ibm.xtools.rmps.web.importer.section_ImportDefinitionEditor001";
var PATHMAPS_UAKEY = "com.ibm.xtools.rmps.web.importer.section_ImportDefinitionEditor002";
var WORKSPACES_UAKEY = "com.ibm.xtools.rmps.web.importer.section_ImportDefinitionEditor003";
var CREATED_VARIABLE = "${created}";

var TRANSFER_TIMEOUT_DEFAULT = "30";
var ANT_EXECUTION_TIMEOUT_DEFAULT = "30";

dojo.declare("rmps.importer.ImportDefinitionEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "rmps-importer-definition",
	
	_deleteButton: null,
	
	_importDefinitionType: null, // object of type ImportDefinitionType
	
	constructor: function() {
		this.uaKey = PAGE_UAKEY;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		this.showLoading();
		
		var deferred = ImportClient.getImportDefinitionForProject(this.getEditorInput().projectId);
		deferred.addBoth(this, function(response) {
			var definition = null;
			if (response instanceof Error) {
				if (response.status && (response.status === 410 || response.status === 404)) {
					this.hideMessage();
				} else {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						errorObject: response,
						closeable: true
					});
					rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
					this.isReadOnly = true;
					return;
				}
			} else {
				this.hideMessage();
				definition = response;
			}
			
			this._init(definition);
		});
	},
	
	_init: function(definition) {
		var report = this.getEditorInput().operationReport;
		
		if (definition) {
			this._definition = definition;
			this.isReadOnly = !report.isAllowed(ProcessConstants.Actions.MODIFY);
			if (this._definition.eclipseWorkspacePath) {
				this._importDefinitionType = ImportDefinitionType.WORKSPACE;
			} else if (this._definition.sampleModelImport) {
				this._importDefinitionType = ImportDefinitionType.SAMPLE;
			} else {
				this._importDefinitionType = ImportDefinitionType.PROJECT;
			}
			
		} else {
			this._definition = new rmps.importer.ImportDefinition();
			this._definition.uri = ImportClient.getImportDefinitionUri(this.getEditorInput().projectId);
			this.isReadOnly = !report.isAllowed(ProcessConstants.Actions.CREATE);
			delete this._importDefinitionType;
		}
		
		if (this.isReadOnly) {
			dojo.addClass(this.containerNode, "disabled");
		} else {
			this.saveButton.domNode.style.display = "";
			dojo.removeClass(this.containerNode, "disabled");
		}
		
		this._createSections();
	},
	
	_createSections: function() {
		if (this._deleteButton && this._deleteButton.parentNode) {
			this._deleteButton.parentNode.removeChild(this._deleteButton);
		}
		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) { 
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		if (this._importDefinitionType == null) {
			if (this.isReadOnly) {
				this.containerNode.appendChild(dojo.doc.createTextNode(messages.noImportDefinition));
			} else {
				this._createTypeSelection();
			}
		} else {
			this._createMainSections();
		}
	},
	
	_createMainSections: function() {
		if (this._definition.eTag) {
			this._create_deleteButton();
		}
		
		if (this._definition.eTag && this._importDefinitionType == ImportDefinitionType.SAMPLE) {
			this._createSampleImportedSection();
		}
		
		this._createGeneralSection();
		
		if (!this._definition.eTag || this._importDefinitionType == ImportDefinitionType.PROJECT) {
			this._createWorkspacesSection();
		}
		
		this._createScheduleSection();
		this._createPathMapsSection();
		this._createAdvancedSection();		
	},
	
	addButtons: function(toolbar) {
		this.inherited(arguments);
		
		// hide save buttons until permission check is complete
		this.saveButton.domNode.style.display = "none";
	},
	
	_create_deleteButton: function() {
		if (this.getEditorInput().operationReport.isAllowed(ProcessConstants.Actions.DELETE)) {
			if (!this._deleteButton) {
				this._deleteButton = dojo.create("button", {className: "deleteButton"});
				this._deleteButton.appendChild(dojo.doc.createTextNode(messages.deleteImportDefinition));
				this.connect(this._deleteButton, "onclick", this._deleteImportDefinition);
			}
			this.containerNode.appendChild(this._deleteButton);
		}
	},
	
	_deleteImportDefinition: function() {
		if (confirm(messages.confirmDelete)) {
			var deferred = ImportClient.deleteImportDefinition(this._definition.uri);
			deferred.addCallback(this, function() {
				delete this._createdSections;
				this.removeDirty(null, DIRTY_GROUP_ID);
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: messages.deleteSuccessful,
					closable: true
				});
				this._init();
			});
			deferred.addErrback(this, function(error) {
				if (error.status === 403) {
					var message = messages.error_delete_permission;
				} else {
					var message = messages.error_delete;
				}
				this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: message,
						closable: true
					});
			});
		}
	},
	
	validate: function(errors) {
		if (this._importDefinitionType == ImportDefinitionType.SAMPLE) {
			// nothing validate
			return;
		}
		if (!this._definition.engineId || this._definition.engineId.length === 0) {
			errors.push(messages.error_no_engine);
		}
		
		if (this._importDefinitionType == ImportDefinitionType.WORKSPACE) {
			if (!this._definition.eclipseWorkspacePath || this._definition.eclipseWorkspacePath.length === 0) {
				errors.push(messages.error_no_eclipse_workspace_path);
			}
		} else {
			if (this._countProjects() === 0) {
				errors.push(messages.error_no_projects);
			}
			
			var duplicateProjectDirNames = [];
			var projectDirNames = {};
			var duplicateProjectAliases = [];
			var projectAliases = {};
			
			var incompleteWorkspace = false;
			for (var i = 0; i < this._definition.workspaces.length; ++i) {
				var workspace = this._definition.workspaces[i];
				
				var hasProject = false;
				for (var j = 0, project; project = workspace.projects[j]; ++j) {
					var projectDirName = project.directoryName;
					var projectAlias = project.alias;
					//project name field doesn't support \ or /
					if ( projectDirName.indexOf("\\") >= 0 || projectDirName.indexOf("/") >= 0) {
						errors.push(dojo.string.substitute(messages.invalid_project_name, [projectDirName]));
					}										
					if (projectDirName.length > 0) {
						hasProject = true;
						if (projectDirNames[projectDirName] != null) {
							if (projectDirNames[projectDirName] === false) {
								projectDirNames[projectDirName] = true;
								duplicateProjectDirNames.push(projectDirName);
							}
						} else {
							projectDirNames[projectDirName] = false;
						}
					}
					
					if (projectAlias.length > 0) {
						hasProject = true;
						if (projectAliases[projectAlias] != null) {
							if (projectAliases[projectAlias] === false) {
								projectAliases[projectAlias] = true;
								duplicateProjectAliases.push(projectAlias);
							}
						} else {
							projectAliases[projectAlias] = false;
						}
						if (projectDirName.length === 0) {
							errors.push(dojo.string.substitute(messages.error_no_project_name, [projectDirName]));
						}
					} 
				}
				
				if (workspace.path.length === 0 && hasProject) {
					if (!incompleteWorkspace) {
						incompleteWorkspace = true;
						errors.push(messages.error_workspace_path_incomplete);
					}
				} else if (workspace.path.length > 0 && !hasProject) {
					errors.push(dojo.string.substitute(messages.error_no_workspace_projects, [workspace.path]));
				}
			}
			
			if (duplicateProjectDirNames.length > 0) {
				duplicateProjectDirNames.sort();
				errors.push(dojo.string.substitute(messages.error_duplicate_project_directory_names, [duplicateProjectDirNames.join(", ")]));
			}
			
			if (duplicateProjectAliases.length > 0) {
				duplicateProjectAliases.sort();
				errors.push(dojo.string.substitute(messages.error_duplicate_project_aliases, [duplicateProjectAliases.join(", ")]));
			}
		}
		
		// check for redeclaration of system path maps
		var spDeferred = new dojo.Deferred();
		errors.push(spDeferred);
		spDeferred.addCallback(this, function(systemPathMaps) {
			var spm = dojo.isArray(systemPathMaps) ? systemPathMaps : [];
			var duplicateVariables = [];
			var variables = {};
			
			for (var i = 0; i < this._definition.pathMaps.length; ++i) {
				var pathMap = this._definition.pathMaps[i];
				
				if (pathMap.variable.length > 0) {
					if (variables[pathMap.variable] != null) {
						if (variables[pathMap.variable] === false) {
							variables[pathMap.variable] = true;
							duplicateVariables.push(pathMap.variable);
						}
					} else {
						variables[pathMap.variable] = false;
					}
				}
			
				if (pathMap.variable.length === 0 && pathMap.value.length !== 0 && dojo.indexOf(spm, pathMap.variable) === -1) {
					errors.push(dojo.string.substitute(messages.error_no_pathMap_variable, [pathMap.value]));
					continue;
				}
				
				if (pathMap.variable.length !== 0 && pathMap.value.length === 0 && dojo.indexOf(spm, pathMap.variable) === -1) {
					errors.push(dojo.string.substitute(messages.error_no_pathMap_value, [pathMap.variable]));
					continue;
				}
				
				var segments = pathMap.value.split("/");
				if (projectDirNames && projectDirNames[segments[0] ? segments[0] : segments[1]] == null) {
					errors.push(dojo.string.substitute(messages.error_pathMap_value_project, [pathMap.variable]));
				}
			}
			
			if (duplicateVariables.length > 0) {
				duplicateVariables.sort();
				errors.push(dojo.string.substitute(messages.error_duplicate_pathMap_variables, [duplicateVariables.join(", ")]));
			}
			
			if (spm.length > 0) {
				var redeclaredPathMaps = [];
				
				for (var i = 0, p; p = this._definition.pathMaps[i]; ++i) {
					if (dojo.indexOf(spm, p.variable) >= 0 && dojo.indexOf(redeclaredPathMaps, p.variable) === -1) {
						redeclaredPathMaps.push(p.variable);
					}
				}
				
				if (redeclaredPathMaps.length > 0) {
					redeclaredPathMaps.sort();
					throw new Error(dojo.string.substitute(messages.error_redeclare_system_pathMap, [redeclaredPathMaps.join(", ")]));
				}
			}
		});
		this._getSystemPathMaps().addBoth(spDeferred, spDeferred.callback);
		
		// schedule
		var scheduleEnabled = !!this._definition.schedule;
		
		var interval = this._definition.scheduleInterval;
		if (interval != parseInt(interval, 10)) {
			errors.push(messages.error_interval);
		}
		
		if (scheduleEnabled) {
		 	if (this._definition.scheduleWeekDays.length === 0) {
				errors.push(messages.error_scheduleDay);
			} 			
			if (this._definition.scheduleBaseline && (!this._definition.scheduleBaselineName || this._definition.scheduleBaselineName.length == 0)) {
				errors.push(messages.error_baselineName);
			}
		}
		
		var transferTimeOut = this._definition.transferTimeOut;
		var transferTimeOutInt = parseInt(transferTimeOut, 10);
		if (transferTimeOut != transferTimeOutInt ||
				transferTimeOutInt <= 0) {
			errors.push(messages.error_transferTimeOut);
		}
		
		var antExecutionTimeOut = this._definition.antExecutionTimeOut;
		var antExecutionTimeOutInt = parseInt(antExecutionTimeOut, 10);
		if (antExecutionTimeOut != antExecutionTimeOutInt ||
				antExecutionTimeOutInt <= 0) {
			errors.push(messages.error_antExecutionTimeOut);
		}
	},
	
	doSave: function(taskMonitor) {
		this.inherited(arguments);
		
		// remove empty data
		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) { 
			if (widget.pruneEmpty) {
				widget.pruneEmpty();
			}
		});
		
		// save mode data
		if (this._importDefinitionType == ImportDefinitionType.WORKSPACE) {
			var workspaces = this._definition.workspaces;
			this._definition.workspaces = [];
		} else {
			var eclipseWorkspacePath = this._definition.eclipseWorkspacePath;
			delete this._definition.eclipseWorkspacePath;
		}
		
		if (this._definition.eTag) {
			var deferred = ImportClient.updateImportDefinition(this._definition, this.getEditorInput().projectId);
		} else {
			var deferred = ImportClient.createImportDefinition(this._definition, this.getEditorInput().projectId);
		}
		
		deferred.addCallback(this, function() {
			this.removeDirty(null, DIRTY_GROUP_ID);
			this._init(this._definition);
		});
		
		deferred.addErrback(this, function(error) {
			// restore mode data
			if (this._importDefinitionType == ImportDefinitionType.WORKSPACE) {
				this._definition.workspaces = workspaces;
			} else {
				this._definition.eclipseWorkspacePath = eclipseWorkspacePath;
			}
			
			if (error.status === 403) {
				return new Error(messages.error_save_permission);
			} else {
				return new Error(messages.error_save);
			}
		});
		
		var task = taskMonitor.beginTask(messages.task_saveDefinition, 1);
		task.handleDeferredWork(deferred, 1);
	},
	
	setMode: function(importDefinitionType) {
		this._importDefinitionType = importDefinitionType;
		if (!this._createdSections) {
			this._createdSections = true;
			this._createMainSections();
		}
	},
	
	// type selection
	_createTypeSelection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._ChoiceSection({editor: this});
		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},

	_createSampleImportedSection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._SampleImportedSection({editor: this});

		if (this._importDefinitionType != ImportDefinitionType.SAMPLE) {
				widget.domNode.style.display = "none";
		}
		widget.connect(this, "setMode", function(importDefinitionType) {
			widget.domNode.style.display = importDefinitionType!=ImportDefinitionType.SAMPLE ? "none" : "";
		});

		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},
	
	// General Section
	_createGeneralSection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._General({editor: this});
		if (this._importDefinitionType == ImportDefinitionType.SAMPLE) {
			widget.domNode.style.display = "none";
		}
		widget.connect(this, "setMode", function(importDefinitionType) {
			widget.domNode.style.display = importDefinitionType==ImportDefinitionType.SAMPLE ? "none" : "";
		});
		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},
	
	// Schedule section
	_createScheduleSection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._Schedule({editor: this});
		if (this._importDefinitionType == ImportDefinitionType.SAMPLE) {
			widget.domNode.style.display = "none";
		}
		widget.connect(this, "setMode", function(importDefinitionType) {
			widget.domNode.style.display = importDefinitionType==ImportDefinitionType.SAMPLE ? "none" : "";
		});
		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},
	
	// Workspaces section
	_createWorkspacesSection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._Workspaces({editor: this});
		if (this._importDefinitionType != ImportDefinitionType.PROJECT) {
			widget.domNode.style.display = "none";
		}
		widget.connect(this, "setMode", function(importDefinitionType) {
			widget.domNode.style.display = importDefinitionType!=ImportDefinitionType.PROJECT ? "none" : "";
		});
		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},
	
	// Path Maps Section
	_createPathMapsSection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._PathMaps({editor: this});
		if (this._importDefinitionType == ImportDefinitionType.SAMPLE) {
			widget.domNode.style.display = "none";
		}
		widget.connect(this, "setMode", function(importDefinitionType) {
			widget.domNode.style.display = importDefinitionType==ImportDefinitionType.SAMPLE ? "none" : "";
		});
		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},
	
	_createAdvancedSection: function() {
		var widget = new rmps.importer.ImportDefinitionEditor._Advanced({editor: this});
		this.containerNode.appendChild(widget.domNode);
		widget.startup();
	},
	
	_countProjects: function() {
		var result = 0;
		for (var i = 0; i < this._definition.workspaces.length; ++i) {
			var projects = this._definition.workspaces[i].projects;
			for (var j = 0; j < projects.length; ++j) {
				if (projects[j].directoryName.length > 0 || projects[j].alias.length > 0) {
					result++;
				}
			}
		}
		return result;
	},
	
	_updateDirty: function(originalValue, newValue, dirtyKey) {
		var isEqual = originalValue == newValue;
		if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
			isEqual = true; // empty arrays
			isEqual = dojo.every(newValue, function(item, idx) {
				return item == originalValue[idx];
			});
		}
		if (isEqual) {
			this.removeDirty(dirtyKey, DIRTY_GROUP_ID);
		} else {
			this.addDirty(dirtyKey, DIRTY_GROUP_ID);
		}
	},
	
	_getSystemPathMaps: function() {
		if (!this._pathMapsDeferred) {
			this._pathMapsDeferred = ImportClient.getSystemPathMaps();
		}
		return this._pathMapsDeferred;
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._ChoiceSection", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	
	templateString: '<div class="choiceSection">' +
				'<label title="${messages.choiceDescriptionDesc}">${messages.choiceDescription}</label>' +
				'<div class="choice">' +
					'<input title="${messages.choice_eclipseWorkspacePathDesc}" id="${id}_c1" type="radio" name="typeChoice" value="1" dojoAttachPoint="_c1" dojoAttachEvent="onclick: _toggleChoice">' +
					'<label title="${messages.choice_eclipseWorkspacePathDesc}" for="${id}_c1">${messages.choice_eclipseWorkspacePath}</label>' +
				'</div>' +
				'<div class="choice">' +
					'<input title="${messages.choice_projectsDesc}" id="${id}_c2" type="radio" name="typeChoice" value="2" dojoAttachPoint="_c2" dojoAttachEvent="onclick: _toggleChoice">' +
					'<label title="${messages.choice_projectsDesc}" for="${id}_c2">${messages.choice_projects}</label>' +
				'</div>' +
			'</div>',
	
	editor: null,
	
	_toggleChoice: function(e) {
		this.editor.setMode(e.target.value);
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._SampleImportedSection", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	messages: messages,
	
	templateString: '<div>' +
	'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" readOnly="${editor.isReadOnly}" collapsible="false">' +
	'<div class="sampleImportedDesc">' + 
	'<span>${messages.sampleImportedDesc}</span>' +
	'</div>' +
	'</div>' +
	'</div>',

	postCreate: function() {
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(messages.sampleImportedTitle));

		this._section.setPrimaryTitle(titleNode);
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._General", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" readOnly="${editor.isReadOnly}" collapsible="false">' +
				'<table class="general">' +
					'<tbody dojoAttachPoint="_tbody"></tbody>' +
				'</table>' +
			'</div>' +
		'</div>',
	
	editor: null,
	
	_definition: null,
	
	postCreate: function() {
		this._definition = this.editor._definition;
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(messages.general));
		
		if (!this.editor.isReadOnly) {
			titleNode.appendChild(createRequiredNode());
		}
		
		this._section.setPrimaryTitle(titleNode);
		
		this._help = [];
		
		var engineInput = this._addGeneralInputRow(
				this._tbody,
				messages.importEngine,
				"engineId",
				true,
				IMPORT_ENGINE_UAKEY,
				dojo.hitch(this, this._enableTestButton))[1];
				
		engineInput.style.width = "25%";
		
		// add engine testing
		this._testButton = dojo.create("button", {"class": "test", title: messages.testTitle}, engineInput.parentNode);
		this._testButton.disabled = !this._definition.engineId;
		this._testButton.appendChild(dojo.doc.createTextNode(messages.test));
		this.connect(this._testButton, "onclick", this._onTestClicked);
		this._testResult = dojo.create("span", null, engineInput.parentNode);
		
		// on enter and on blur of the engine input we should
		this.connect(engineInput, "onblur", this._testEngine);
		this.connect(engineInput, "onkeypress", function(e) {
			if (e.keyCode === dojo.keys.ENTER) {
				this._testEngine();
			}
		});
		
		var antScriptInput = this._addGeneralInputRow(
				this._tbody,
				messages.antScriptPath,
				"antScriptPath",
				false,
				ANT_SCRIPT_UAKEY)[1];
				
		antScriptInput.style.width = "50%";
		
		var workspaceArr = this._addGeneralInputRow(
				this._tbody,
				messages.eclipseWorkspacePath,
				"eclipseWorkspacePath",
				true,
				ECLIPSE_WORKSPACE_PATH_UAKEY);
				
		workspaceArr[1].style.width = "50%";
		if (this.editor._importDefinitionType != ImportDefinitionType.WORKSPACE) {
			workspaceArr[0].style.display = "none";
		}
		this.connect(this.editor, "setMode", function(importDefinitionType) {
			workspaceArr[0].style.display = importDefinitionType==ImportDefinitionType.WORKSPACE ? "" : "none";
		});
		
		if (!this.editor.isReadOnly && (!this._definition.engineId || this._definition.engineId.length === 0)) {
			this._section.edit();
			engineInput.focus();
		}
	},
	
	startup: function() {
		this.inherited(arguments);
		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	},
	
	destroy: function() {
		dojo.forEach(this._help, function(help) {
			help.destroyRecursive();
		});
		this.inherited(arguments);
	},
	
	_onTestClicked: function(e) {
		dojo.stopEvent(e);
		this._testEngine();
	},
	
	_testEngine: function() {
		if (this._testButton.disabled) {
			return;
		}
		this._testButton.disabled = true;
		this._testResult.className = "testing";
		dojo.place(dojo.doc.createTextNode(messages.testing), this._testResult, "only");
		var deferred = ImportClient.checkEngine(this._definition.engineId);
		deferred.addBoth(this, function(checkEngine) {
			this._testButton.disabled = false;
			if (!checkEngine || checkEngine instanceof Error) {
				this._testResult.className = "notRunning";
				dojo.place(dojo.doc.createTextNode(messages.engineNotRunning), this._testResult, "only");
			} else {
				this._testResult.className = "running";
				dojo.place(dojo.doc.createTextNode(
					dojo.string.substitute(messages.engineRunning,
							[dojo.date.locale.format(checkEngine.responseTime, {formatLength: "medium"})])),
					this._testResult, "only");
			}
		});
	},
	
	_enableTestButton: function(engineId) {
		this._testButton.disabled = !engineId;
		this._testResult.innerHTML = "";
	},
	
	_addGeneralInputRow: function(tbody, label, field, required, uaKey, onInput) {
		var originalValue = this._definition[field] || "";
		
		var tr = dojo.create("tr");
		
		// add label column
		var labelTd = dojo.create("td", {className: "property"}, tr);
		var labelContainer = dojo.create("div", null, labelTd);
		var labelNode = dojo.create("span", {className: "label"}, labelContainer);
		
		if (!this.editor.isReadOnly && required) {
			var cshNode = dojo.create("span", null, labelNode);
			labelNode.appendChild(createRequiredNode());
		} else {
			var cshNode = labelNode;
		}
		
		cshNode.appendChild(dojo.doc.createTextNode(label));
		
		// CSH
		if (uaKey) {
			this._help.push(new jazz.app.ua.TextHoverHelp({uaKey: uaKey, contentNode: cshNode}));
		}
		
		// add value input column
		var valueTd = dojo.create("td", null, tr);
		var container = dojo.create("div", null, valueTd);
		
		var preview = dojo.create("span", {className: "preview"}, container);
		preview.appendChild(dojo.doc.createTextNode(originalValue));
		
		var input = dojo.create("input", {className: "editable", type: "text", value: originalValue}, container);
		
		tbody.appendChild(tr);
		labelTd = valueTd = container = tbody = null;
		
		if (!this.editor.isReadOnly) {
			connectOnInput(input, this, function() {
				this._definition[field] = dojo.trim(input.value);
				this.editor._updateDirty(originalValue, this._definition[field], input);
				preview.innerHTML = "";
				preview.appendChild(dojo.doc.createTextNode(this._definition[field]));
				if (onInput) {
					onInput(this._definition[field]);
				}
			});
		}
		
		return [tr, input];
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._Workspaces", [dijit._Widget, dijit._Templated], {
	
	uaKey: WORKSPACES_UAKEY,
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" readOnly="${editor.isReadOnly}" uaKey="${uaKey}">' +
				'<div class="workspaceDesc">' +
					'<label>${messages.importProjectsSectionDesc}</label>' +
				'</div>' +
		'</div>',
	
	editor: null,
	
	_definition: null,
	
	postCreate: function() {
		this._definition = this.editor._definition;
		this._children = [];
		
		// sort workspaces
		this._definition.workspaces.sort(function(a, b) {
			var la = a.path.toLowerCase();
			var lb = b.path.toLowerCase();
			return (la > lb) ? 1 : ((la < lb) ? -1 : 0);
		});
			
		this._updateProjectCount();
		
		var titleNode = dojo.create("span");
		titleNode.appendChild(dojo.doc.createTextNode(messages.projects));
		
		if (!this.editor.isReadOnly) {
			titleNode.appendChild(createRequiredNode());
		}
		
		this._section.setPrimaryTitle(titleNode);
		
		// create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.workspacePath, width: (this.editor.isReadOnly ? "50%" : "45%")},
					{label: "", className: "actions"},
					{label: messages.projectDirectoryName},
					{label: messages.projectAlias},
					{label: "", className: "actions"}
				]
		});
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		
		if (!this.editor.isReadOnly) {
			// create Add command
			this._section.addAction(messages.add, messages.addWorkspace, dojo.hitch(this, function() {
				this._section.edit();
				this._addWorkspaceRow();
			}));
		}
		
		// init
		for (var i = this._definition.workspaces.length - 1; i >= 0; --i) {
			this._addWorkspaceRow(this._definition.workspaces[i]);
		}
	},
	
	getChildren: function() {
		return this._children.slice();
	},
	
	_updateProjectCount: function() {
		this._section.setSecondaryTitle('(' + this.editor._countProjects() + ')');
	},
	
	_addWorkspaceRow: function(data) {
		var row = new rmps.importer.ImportDefinitionEditor._WorkspaceRow({
			workspaceSection: this,
			editor: this.editor,
			workspace: data
		});
		this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		row.startup();
		
		this._children.push(row);
		var handle = this.connect(row, "uninitialize", function() {
			this.disconnect(handle);
			jazz.util.array.removeByReference(this._children, row);
		});
		
		if (!data) {
			row.focus();
		}
	},
	
	pruneEmpty: function() {
		dojo.forEach(this.getChildren(), function(row) { 
			row.pruneEmpty();
			if (!row.workspace.path && row.workspace.projects.length === 0) {
				row.remove();
				row.destroyRecursive();
			}
		});
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._WorkspaceRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_pathPreview"></span>' +
					'<input class="editable" style="width: 100%;" dojoAttachPoint="_pathInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 26px;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	workspaceSection: null,
	editor: null,
	workspace: null,
	
	_definition: null,
	_originalWorkspace: null,
	
	postCreate: function() {
		this._definition = this.editor._definition;
		this._children = [];
		
		if (!this.workspace) {
			this.workspace = {
				path: "",
				projects: []
			};
			this._definition.workspaces.push(this.workspace);
			
		} else {
			// sort projects
			this.workspace.projects.sort(function(a, b) {
				var la = a.directoryName.toLowerCase();
				var lb = b.directoryName.toLowerCase();
				return (la > lb) ? 1 : ((la < lb) ? -1 : 0);
			});
		
			this._pathInput.value = this.workspace.path;
			this._pathPreview.appendChild(dojo.doc.createTextNode(this.workspace.path));
			this._pathInput.focus();
			
			this._originalWorkspace = dojo.clone(this.workspace);
		}
		
		if (!this.editor.isReadOnly) {
			// add remove button
			this.toolbar.addButton(this, function(e) {
						dojo.stopEvent(e);
						this.remove();
						this.destroyRecursive();
					}, {
					label: messages.removeWorkspace,
					imgSrc: dojo.moduleUrl("rmps.importer", "graphics/icons/remove.gif")
				});
			
			connectOnInput(this._pathInput, this, function() {
				this.workspace.path = dojo.trim(this._pathInput.value);
				this.editor._updateDirty(this._originalWorkspace ? this._originalWorkspace.path : "", this.workspace.path, this._pathInput);
				this._pathPreview.innerHTML = "";
				this._pathPreview.appendChild(dojo.doc.createTextNode(this.workspace.path));
			});
		}
	},
	
	focus: function() {
		this._pathInput.focus();
	},
	
	getChildren: function() {
		return this._children.slice();
	},
	
	startup: function() {
		this.inherited(arguments);
		
		if (this.workspace.projects.length === 0) {
			this._addWorkspaceProject();
		} else {
			for (var i = this.workspace.projects.length - 1; i >= 0; --i) {
				this._addWorkspaceProject(this.workspace.projects[i]);
			}
		}
	},
	
	_addWorkspaceProject: function(data, ref) {
		var row = new rmps.importer.ImportDefinitionEditor._WorkspaceProjectRow({
			workspaceSection: this.workspaceSection,
			workspaceRow: this,
			editor: this.editor,
			project: data
		});
		
		row.__childNodes = [];
		for (var i = 0, child; child = row.domNode.childNodes[i]; ++i) {
	        row.__childNodes.push(child);
	    }
		
		if (ref) {
			if (ref === this._children[0]) {
				dojo.place(row.domNode, this.domNode, "after");
			} else {
				dojo.place(row.domNode, ref.domNode, "after");
			}
		} else {
			var firstChild = this._children[0];
			if (firstChild) {
			    for (var i = 0, child; child = firstChild.__childNodes[i]; ++i) {
			        firstChild.domNode.appendChild(child);
			    }
	    		dojo.place(firstChild.domNode, this.domNode, "after");
			}
			
		    for (var i = 0, child; child = row.__childNodes[i]; ++i) {
		        this.domNode.appendChild(child);
		    }
		}
		
		if (this._children.length === 0) {
			row.setRemoveButtonEnabled(false);
		} else if (this._children.length === 1) {
			this._children[0].setRemoveButtonEnabled(true);
		}
		
		firstChild = i = child = null;
		
		this._updateRowSpan();
		row.startup();
		
		if (ref) {
			this._children.splice(dojo.indexOf(this._children, ref) + 1, 0, row);
		} else {
			this._children.splice(0, 0, row);
		}
		
		var handle = this.connect(row, "uninitialize", function() {
			this.disconnect(handle);
			if (row === this._children[0]) {
			    for (var i = 0, child; child = row.__childNodes[i]; ++i) {
			        row.domNode.appendChild(child);
			    }
			    var nextChild = this._children[1];
			    if (!this._beingDestroyed && nextChild) {
			        for (var i = 0, child; child = nextChild.__childNodes[i]; ++i) {
				        this.domNode.appendChild(child);
				    }
				    nextChild.domNode.parentNode.removeChild(nextChild.domNode);
			    }
			}
			jazz.util.array.removeByReference(this._children, row);
			this._updateRowSpan();
			if (this._children.length === 1) {
				this._children[0].setRemoveButtonEnabled(false);
			}
		});
		
		if (!data) {
			row.focus();
		}
	},
	
	_updateRowSpan: function() {
		var rs = this.workspace.projects.length || 1;
		// only update the rowspan for the first two cells
		for (var i = 0; i < 2; ++i) {
			this.domNode.childNodes[i].rowSpan = rs;
		}
	},
	
	remove: function() {
		dojo.forEach(this._children, function(row) {
			row.remove();
		});
		jazz.util.array.removeByReference(this._definition.workspaces, this.workspace);
		
		if (this._originalWorkspace) {
			this.editor.addDirty(this._originalWorkspace, DIRTY_GROUP_ID);
		}
		
		this.editor.removeDirty(this._pathInput, DIRTY_GROUP_ID);
	},
	
	pruneEmpty: function() {
		dojo.forEach(this.getChildren(), function(row) { 
			if (!row.project.directoryName) {
				row.remove();
				row.destroyRecursive();
			}
		});
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._WorkspaceProjectRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_namePreview"></span>' +
					'<input class="editable" style="width: 100%;" dojoAttachPoint="_nameInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_aliasPreview"></span>' +
					'<input class="editable" style="width: 100%;" dojoAttachPoint="_aliasInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 53px;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	workspaceSection: null,
	workspaceRow: null,
	editor: null,
	project: null,
	
	_definition: null,
	_originalProject: null,
		
	postCreate: function() {
		this._definition = this.editor._definition;
		
		if (!this.project) {
			this.project = {
				directoryName: "",
				alias: ""
			};
			this.workspaceRow.workspace.projects.push(this.project);
			
		} else {
			this._nameInput.value = this.project.directoryName;
			this._namePreview.appendChild(dojo.doc.createTextNode(this.project.directoryName));
				
			if (this.project.alias != this.project.directoryName) {
				this._aliasInput.value = this.project.alias;
				this._aliasPreview.appendChild(dojo.doc.createTextNode(this.project.alias));
			} else {
				this.project.alias = "";
			}
			this._nameInput.focus();
			
			this._originalProject = dojo.clone(this.project);
		}
		
		if (!this.editor.isReadOnly) {
			// add project button
			this.toolbar.addButton(this, function(e) {
						dojo.stopEvent(e);
						this.workspaceSection._section.edit();
						this.workspaceRow._addWorkspaceProject(null, this);
					}, {
					label: messages.addProject,
					imgSrc: dojo.moduleUrl("rmps.importer", "graphics/icons/add_project.gif")
				});
				
			// add remove button
			this._removeBtn = this.toolbar.addButton(this, function(e) {
						dojo.stopEvent(e);
						this.remove();
						this.destroyRecursive();
					}, {
					label: messages.removeProject,
					imgSrc: dojo.moduleUrl("rmps.importer", "graphics/icons/remove.gif")
				});
			
			
			connectOnInput(this._nameInput, this, function() {
				this.project.directoryName = dojo.trim(this._nameInput.value);
				this.editor._updateDirty(this._originalProject ? this._originalProject.directoryName : "", this.project.directoryName, this._nameInput);
				this.workspaceSection._updateProjectCount();
				this._namePreview.innerHTML = "";
				this._namePreview.appendChild(dojo.doc.createTextNode(this.project.directoryName));
				this._updateAliasName();
			});
			
			connectOnInput(this._aliasInput, this, function() {
				this.project.alias = dojo.trim(this._aliasInput.value);
				this.editor._updateDirty(this._originalProject ? this._originalProject.alias : "", this.project.alias, this._aliasInput);
				this.workspaceSection._updateProjectCount();
				this._aliasPreview.innerHTML = "";
				this._aliasPreview.appendChild(dojo.doc.createTextNode(this.project.alias));
			});
			
			// connect blur last
			this.connect(this._aliasInput, "onblur", this._updateAliasName);
		}
		
		this._updateAliasName();
	},
	
	focus: function() {
		this._nameInput.focus();
	},
	
	setRemoveButtonEnabled: function(enable) {
		if (enable) {
			this.toolbar.enableButton(this._removeBtn);
		} else {
			this.toolbar.disableButton(this._removeBtn);
		}
	},
	
	_updateAliasName: function() {
		if (this.project.alias.length > 0) {
			return;
		}
		this._aliasInput.value = this.project.directoryName;
		this._aliasPreview.innerHTML = "";
		this._aliasPreview.appendChild(dojo.doc.createTextNode(this.project.directoryName));
	},
	
	remove: function() {
		jazz.util.array.removeByReference(this.workspaceRow.workspace.projects, this.project);
		this.workspaceSection._updateProjectCount();
		
		if (this._originalProject) {
			this.editor.addDirty(this._originalProject, DIRTY_GROUP_ID);
		}
		
		this.editor.removeDirty(this._nameInput, DIRTY_GROUP_ID);
		this.editor.removeDirty(this._aliasInput, DIRTY_GROUP_ID);
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._PathMaps", [dijit._Widget, dijit._Templated], {
	
	uaKey: PATHMAPS_UAKEY,
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" primaryTitle="${messages.pathMaps}" isCollapsed=true readOnly="${editor.isReadOnly}" uaKey="${uaKey}"></div>' +
		'</div>',
	
	editor: null,
	
	_definition: null,
	
	_pathMaps : null,
	
	
	postCreate: function() {
		this._definition = this.editor._definition;
		
		// sort path maps
		this._definition.pathMaps.sort(function(a, b) {
			var la = a.variable.toLowerCase();
			var lb = b.variable.toLowerCase();
			return (la > lb) ? 1 : ((la < lb) ? -1 : 0);
		});
		
		
		// create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.variable, width: "50%"},
					{label: messages.value},
					{label: "", className: "actions"}
				]
		});
		//add tooltip
		var headerEntered = false;
	    var oldHeaderTooltip = null;
		this.connect(this._table._headerRow, "onmouseover", function(){
		    if (!headerEntered){
		    	oldHeaderTooltip = this._section.containerNode.title;
		    }
		    headerEntered = true;
		    this._section.containerNode.title = dojo.string.substitute(messages.pathMapsHeaderTooltip, 
		    				[messages.projectDirectoryName, messages.projects]);
		});
		this.connect(this._table._headerRow, "onmouseout", function(){
			headerEntered = false;
			this._section.containerNode.title = oldHeaderTooltip;
		});
		
		this._table.containerNode = this._table.tbody;
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		
		// create Add command
		if (!this.editor.isReadOnly) {
			this._section.addAction(messages.add, messages.addPathMap, dojo.hitch(this, function() {
				this._section.edit();
				this._addPathMapRow();
			}));
		}
		
		// init
		for (var i = this._definition.pathMaps.length - 1; i >= 0; --i) {
			this._addPathMapRow(this._definition.pathMaps[i]);
		}
		
		// add system defined path maps
		var loadingTr = dojo.create("tr", {className: "systemPathMap"});
		dojo.create("span", null,
			dojo.create("td", {colSpan: "3"}, loadingTr)).appendChild(dojo.doc.createTextNode(messages.loadingSystemDefinedPathMaps));
		this._table.tbody.appendChild(loadingTr);
		this.editor._getSystemPathMaps().addCallback(this, function(pathMaps) {
			dojo.destroy(loadingTr);
			delete loadingTr;
			for (var i = 0, p; p = pathMaps[i]; ++i) {
				var row = new rmps.importer.ImportDefinitionEditor._SystemPathMapRow({pathMap: p});
				this._table.tbody.appendChild(row.domNode);
				row.startup();
			}
			this._pathMaps = pathMaps;
			this._updatePathMapCount();
		});
	},
	
	_countUserDefinedPathMaps: function() {
		var result = 0;
		var pathMaps = this._definition.pathMaps;
		for (var i = 0; i < pathMaps.length; ++i) {
			if (pathMaps[i].variable.length > 0 || pathMaps[i].value.length) {
				result++;
			}
		}
		return result;
	},
	
	_updatePathMapCount: function() {
		var counter = this._countUserDefinedPathMaps() + this._pathMaps.length;
		this._section.setSecondaryTitle('(' + counter + ')');
	},
	
	_addPathMapRow: function(data) {
		var row = new rmps.importer.ImportDefinitionEditor._PathMapRow({
			pathMapSection: this,
			editor: this.editor,
			pathMap: data
		});
		this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		row.startup();
		
		if (!data) {
			row.focus();
		}
	},
	
	pruneEmpty: function() {
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) { 
			if (!row.isSystem && !row.pathMap.variable && !row.pathMap.value) {
				row.remove();
				row.destroyRecursive();
			}
		});
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._SystemPathMapRow", [dijit._Widget, dijit._Templated], {
	
	isSystem: true,
	pathMap: null,
	messages: messages,
	
	templateString: '<tr class="systemPathMap">' +
			'<td>' +
				'<div>' +
					'<span>${pathMap}</span>' +
				'</div>' +
			'</td>' +
			'<td colspan="2">' +
				'<div>' +
					'<span>${messages.systemDefined}</span>' +
				'</div>' +
			'</td>' +
		'</tr>'
});

dojo.declare("rmps.importer.ImportDefinitionEditor._PathMapRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_variablePreview"></span>' +
					'<input class="editable" style="width: 100%;" dojoAttachPoint="_variableInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_valuePreview"></span>' +
					'<input class="editable" style="width: 100%;" dojoAttachPoint="_valueInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 26px;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	pathMapSection: null,
	editor: null,
	pathMap: null,
	
	_definition: null,
	_originalPathMap: null,
	
	postCreate: function() {
		this._definition = this.editor._definition;
		
		if (!this.pathMap) {
			this.pathMap = {
				variable: "",
				value: ""
			};
			this._definition.pathMaps.push(this.pathMap);
			
		} else {
			this._variableInput.value = this.pathMap.variable;
			this._variablePreview.appendChild(dojo.doc.createTextNode(this.pathMap.variable));
			this._valueInput.value = this.pathMap.value;
			this._valuePreview.appendChild(dojo.doc.createTextNode(this.pathMap.value));
			this._variableInput.focus();
			
			this._originalPathMap = dojo.clone(this.pathMap);
		}
		
		// add remove button
		if (!this.editor.isReadOnly) {
			this.toolbar.addButton(this, function(e) {
						dojo.stopEvent(e);
						this.remove();
						this.destroyRecursive();
					}, {
					label: messages.removePathMap,
					imgSrc: dojo.moduleUrl("rmps.importer", "graphics/icons/remove.gif")
				});
			
			connectOnInput(this._variableInput, this, function() {
				this.pathMap.variable = dojo.trim(this._variableInput.value);
				this.editor._updateDirty(this._originalPathMap ? this._originalPathMap.variable : "", this.pathMap.variable, this._variableInput);
				this.pathMapSection._updatePathMapCount();
				this._variablePreview.innerHTML = "";
				this._variablePreview.appendChild(dojo.doc.createTextNode(this.pathMap.variable));
			});
			
			connectOnInput(this._valueInput, this, function() {
				this.pathMap.value = dojo.trim(this._valueInput.value);
				this.editor._updateDirty(this._originalPathMap ? this._originalPathMap.value : "", this.pathMap.value, this._valueInput);
				this.pathMapSection._updatePathMapCount();
				this._valuePreview.innerHTML = "";
				this._valuePreview.appendChild(dojo.doc.createTextNode(this.pathMap.value));
			});
		}
	},
	
	focus: function() {
		this._variableInput.focus();
	},
	
	remove: function() {
		jazz.util.array.removeByReference(this._definition.pathMaps, this.pathMap);
		this.pathMapSection._updatePathMapCount();
		
		if (this._originalPathMap) {
			this.editor.addDirty(this._originalPathMap, DIRTY_GROUP_ID);
		}
		this.editor.removeDirty(this._valueInput, DIRTY_GROUP_ID);
		this.editor.removeDirty(this._variableInput, DIRTY_GROUP_ID);
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._Schedule", [dijit._Widget, dijit._Templated], {
	
	uaKey: SCHEDULE_UAKEY,
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" primaryTitle="${messages.schedule}" uaKey="${uaKey}">' +
				'<div class="schedule">' +
					'<div class="item">' +
						'<input id="${id}_eCb" type="checkbox" dojoAttachPoint="_enabledCb" dojoAttachEvent="onclick: _onEnabled">' +
						'<label for="${id}_eCb">${messages.enableSchedule}</label>' +
					'</div>' +
					'<table>' +
						'<tr>' +
							'<td>' +
								'<fieldset>' +
								'<legend>${messages.importTime}</legend>' +
								'<div class="item">' +
									'<input dojoAttachPoint="_intervalRadio" type="radio" name="scheduleType" value="interval" dojoAttachEvent="onclick: _onInterval"> ${messages.interval} ' +
									'<input dojoAttachPoint="_intervalInput" type="text" maxlength="4" style="width: 50px;">' +
									'<br>' +
									'<input dojoAttachPoint="_specificRadio" type="radio" name="scheduleType" value="specific" dojoAttachEvent="onclick: _onSpecific"> ${messages.at} ' +
									'<select dojoAttachPoint="_hoursSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> : ' +
									'<select dojoAttachPoint="_minutesSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> ' +
									'<select dojoAttachPoint="_ampmSelect" dojoAttachEvent="onchange: _onScheduleTimeChanged"></select> ${messages.gmt}' +
								'</div>' +
								'</fieldset>' +
							'</td>' +
							'<td rowspan="2">' +
								'<fieldset style="height: 140px; padding-top: 10px;">' +
								'<legend>${messages.importDays}</legend>' +
								'<div>' +
									'<select dojoAttachPoint="_daysSelect" dojoAttachEvent="onchange: _onDaysChanged" multiple="true" size="7" style="margin-right: 10px; overflow: hidden; float: left;"></select>' +
									'<div style="float: left;">' +
										'<button dojoAttachPoint="_selectAllBtn" style="width: 100%; margin-bottom: 4px;" dojoAttachEvent="onclick: _onSelectAllDays">${messages.selectAll}</button><br>' +
										'<button dojoAttachPoint="_deselectAllBtn" style="width: 100%;" dojoAttachEvent="onclick: _onDeselectAllDays">${messages.deselectAll}</button>' +
									'</div>' +
								'</div>' +
								'</fieldset>' +
							'</td>' +
						'</tr>' +
						'<tr>' +
							'<td>' +
								'<fieldset>' +
								'<legend>${messages.scheduleBaseline}</legend>' +
								'<div class="item">' +
									'<input dojoType="dijit.form.CheckBox" dojoAttachPoint="_baselineCheck" dojoAttachEvent="onClick: _onBaselineChanged"/>' +
									'<label>${messages.enableBaseline}</label>' +
									'<br>' +								
									'<span dojoAttachPoint="_baselineNameLabel" style="margin-left:20px;">${messages.baselineName}</span>' +								
									'<input dojoAttachPoint="_baselineName" style="width: 260px;">' +
								'</div>' +
								'</fieldset>' +
							'</td>' +
						'</tr>' +
					'</table>' +					
				'</div>' +
			'</div>' +
		'</div>',
	
	
	editor: null,
	
	_section: null,
	_definition: null,
	_originalDefinition: null,
	_help: null,
	
	postCreate: function() {
		if (dojo.isIE < 8) {
			this._selectAllBtn.style.width = "";
			this._deselectAllBtn.style.width = "";
		}
		
		this._definition = this.editor._definition;
		this._originalDefinition = dojo.clone(this._definition);
		this._help = [];
		
		// set radios
		if (!this._originalDefinition.schedule || this._originalDefinition.schedule === rmps.importer.ImportDefinitionSchedule.interval) {
			this._intervalRadio.checked = true;
		} else if (this._originalDefinition.schedule === rmps.importer.ImportDefinitionSchedule.time) {
			this._specificRadio.checked = true;
		}
		
		// interval
		if (this._originalDefinition.scheduleInterval === null) {
			this._originalDefinition.scheduleInterval = this._definition.scheduleInterval = 30;
		}
		this._intervalInput.value = this._originalDefinition.scheduleInterval;
		
		// populate hours
		if (!this._originalDefinition.scheduleTime) {
			this._originalDefinition.scheduleTime = this._definition.scheduleTime = ["01", "00"];
		}
		
		var hour24 = this._parseHour(this._originalDefinition.scheduleTime[0]);
		var hour =  hour24 % 12;
		for (var i = 1; i <= 12; ++i) {
			var label = i + "";
			if (i < 10) {
			 	label = "0" + i;
			}
			var value = label;
			if (i === 12) {
				value = "00";
			}
			this._createOption(this._hoursSelect, label, value, this._parseHour(value) == hour);
		}
		
		// populate minutes
		var minutes = this._originalDefinition.scheduleTime[1];
		for (var i = 0; i < 60; ++i) {
			var value = i;
			if (i < 10) {
			 	value = "0" + i;
			}
			this._createOption(this._minutesSelect, value, value, value == minutes);
		}
		
		// populate AM / PM
		var isPM = hour24 >= 12;
		this._createOption(this._ampmSelect, dojo.date.locale._getGregorianBundle()["am"], "am", !isPM);
		this._createOption(this._ampmSelect, dojo.date.locale._getGregorianBundle()["pm"], "pm", isPM);
		
		// populate week days
		var weekDays = dojo.date.locale.getNames("days", "wide");
		// move sunday to the end
		weekDays.push(weekDays[0]);
		weekDays.shift();
		
		for (var i = 0; i < weekDays.length; ++i) {
			this._createOption(this._daysSelect, weekDays[i], i + 1, dojo.some(this._originalDefinition.scheduleWeekDays, function(item) {
				return item == (i + 1);
			}));
		}
		
		//populate baseline info
		if (this._originalDefinition.scheduleBaseline) {
			this._baselineCheck.attr("value", true);
		}
		this._baselineName.value = this._originalDefinition.scheduleBaselineName;		
		//add a required node to the label of this input box
		this._baselineName.parentNode.insertBefore(createRequiredNode(), this._baselineName);
				// CSH
			this._help.push(new jazz.app.ua.TextHoverHelp({uaKey: SNAPSHOT_NAME_UAKEY, contentNode: this._baselineNameLabel}));
		
		// enabled
		var enabled = !!this._originalDefinition.schedule;
		this._enabledCb.checked = enabled;
		this._updateEnablement(enabled && !this.editor.isReadOnly);
		if (this.editor.isReadOnly) {
			this._enabledCb.disabled = true;
		}
		
		// event listener for interval input
		var originalScheduleInterval = this._definition.scheduleInterval;
		connectOnInput(this._intervalInput, this, this._onIntervalChanged);
		
		// event listener for baseline name input
		connectOnInput(this._baselineName, this, this._onBaselineNameChanged);
	},
	
	startup: function() {
		this.inherited(arguments);
		if (dojo.isIE < 8) {
			var sow = this._selectAllBtn.offsetWidth;
			var dow = this._deselectAllBtn.offsetWidth;
			if (sow < dow) {
				this._selectAllBtn.style.width = dow + "px";
			} else {
				this._deselectAllBtn.style.width = sow + "px";
			}
		}

		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	},
	
	_createOption: function(selectNode, label, value, selected) {
		var option = dojo.create("option");
		option.value = value;
		option.appendChild(dojo.doc.createTextNode(label));
		if (selected) {
			option.selected = true;
		}
		selectNode.appendChild(option);
	},
	
	_updateEnablement: function(enabled) {
		if (enabled) {
			this._section.setSecondaryTitle(messages.scheduleEnabled);
			
			if (dojo.isIE < 8 || dojo.isChrome) {
				var options = this._daysSelect.options;
				for (var i = 0, option; option = options[i]; ++i) {
					if (option.selected) {
						dojo.removeClass(option, "disabledSelected");
					}
				}
			}
			
			this._intervalRadio.disabled = false;
			this._specificRadio.disabled = false;
			this._daysSelect.disabled = false;
			this._selectAllBtn.disabled = false;
			this._deselectAllBtn.disabled = false;
			this._baselineCheck.disabled = false;
		
			if (this._baselineCheck.checked) {
				this._baselineName.disabled = false;
			} else {
				this._baselineName.disabled = true;
			}
			
			if (this._intervalRadio.checked) {
				this._intervalInput.disabled = false;
				this._hoursSelect.disabled = true;
				this._minutesSelect.disabled = true;
				this._ampmSelect.disabled = true;
			} else {
				this._intervalInput.disabled = true;
				this._hoursSelect.disabled = false;
				this._minutesSelect.disabled = false;
				this._ampmSelect.disabled = false;
			}
		} else {
			this._section.setSecondaryTitle(messages.scheduleDisabled);
			this._intervalRadio.disabled = true;
			this._specificRadio.disabled = true;
			this._intervalInput.disabled = true;
			this._hoursSelect.disabled = true;
			this._minutesSelect.disabled = true;
			this._ampmSelect.disabled = true;
			this._daysSelect.disabled = true;
			this._selectAllBtn.disabled = true;
			this._deselectAllBtn.disabled = true;
			this._baselineCheck.disabled = true;
			this._baselineName.disabled = true;
			
			if (dojo.isIE < 8 || dojo.isChrome) {
				var options = this._daysSelect.options;
				for (var i = 0, option; option = options[i]; ++i) {
					if (option.selected) {
						dojo.addClass(option, "disabledSelected");
					}
				}
			}
			
		}
	},
	
	_onEnabled: function() {		
		if (this._enabledCb.checked) {
			if (this._intervalRadio.checked) {
				this._definition.schedule = rmps.importer.ImportDefinitionSchedule.interval;
			} else {
				this._definition.schedule = rmps.importer.ImportDefinitionSchedule.time;
			} 
		} else {
			this._definition.schedule = null;
		}
		
		this._updateEnablement(this._enabledCb.checked);
		this.editor._updateDirty(this._originalDefinition.schedule, this._definition.schedule, this._enabledCb);
	},
	
	_onInterval: function() {
		this._updateEnablement(true);
		this._definition.schedule = rmps.importer.ImportDefinitionSchedule.interval;
		this.editor._updateDirty(this._originalDefinition.schedule, this._definition.schedule, this._enabledCb);
	},
	
	_onSpecific: function() {
		this._updateEnablement(true);
		this._definition.schedule = rmps.importer.ImportDefinitionSchedule.time;
		this.editor._updateDirty(this._originalDefinition.schedule, this._definition.schedule, this._enabledCb);
	},
	
	_onSelectAllDays: function() {
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = true;
		}
		this._onDaysChanged();
	},
	
	_onDeselectAllDays: function() {
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			option.selected = "";
		}
		this._onDaysChanged();
	},
	
	_onIntervalChanged: function() {
		this._definition.scheduleInterval = this._intervalInput.value;
		this.editor._updateDirty(this._originalDefinition.scheduleInterval, this._definition.scheduleInterval, this._intervalInput);
	},
	
	_onScheduleTimeChanged: function() {
		var hour = this._hoursSelect.options[this._hoursSelect.selectedIndex].value;
		var minutes = this._minutesSelect.options[this._minutesSelect.selectedIndex].value;
		if (this._ampmSelect.options[this._ampmSelect.selectedIndex].value === "pm") {
			hour = this._parseHour(hour) + 12 + "";
		}
		this._definition.scheduleTime = [hour, minutes];
		
		this.editor._updateDirty(this._originalDefinition.scheduleTime, this._definition.scheduleTime, this._hoursSelect);
	},
	
	_onDaysChanged: function() {
		var days = [];
		var options = this._daysSelect.options;
		for (var i = 0, option; option = options[i]; ++i) {
			if (option.selected) {
				days.push(i + 1);
			}
		}
		
		this._definition.scheduleWeekDays = days;
		this.editor._updateDirty(this._originalDefinition.scheduleWeekDays, this._definition.scheduleWeekDays, this._daysSelect);
	},
	
	_onBaselineChanged: function() {
		this._updateEnablement(true);
		this._definition.scheduleBaseline = !!this._baselineCheck.checked;
		this.editor._updateDirty(this._originalDefinition.scheduleBaseline, this._definition.scheduleBaseline, this._baselineCheck);
	},
	
	_onBaselineNameChanged: function() {
		this._definition.scheduleBaselineName = this._baselineName.value
		if (this._definition.scheduleBaselineName.indexOf(CREATED_VARIABLE) == -1) {
			this._definition.scheduleBaselineName = this._definition.scheduleBaselineName + '_' + CREATED_VARIABLE;
		}
		this.editor._updateDirty(this._originalDefinition.scheduleBaselineName, this._definition.scheduleBaselineName, this._baselineName);
	},
	
	_parseHour: function(hour) {
		return parseInt(hour.replace(/^0/, ''));
	}
});

dojo.declare("rmps.importer.ImportDefinitionEditor._Advanced", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" isCollapsed=true primaryTitle = "${messages.advanced}">' +
			'</div>' +
		'</div>',
		
	editor: null,
	
	_definition: null,
	
	_originalDefinition: null,
	
	postCreate: function() {
		this._definition = this.editor._definition;
		this._originalDefinition = dojo.clone(this._definition);
		
		if (this._originalDefinition.transferTimeOut == null) {
			this._originalDefinition.transferTimeOut = this._definition.transferTimeOut = TRANSFER_TIMEOUT_DEFAULT;
		}
		
		if (this._originalDefinition.antExecutionTimeOut == null) {
			this._originalDefinition.antExecutionTimeOut = this._definition.antExecutionTimeOut = ANT_EXECUTION_TIMEOUT_DEFAULT;
		}
		
		// create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.property, width: ("50%")},
					{label: messages.currentValue},
					{label: messages.defaultValue},
				]
		});
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		var transferTimeOutRow = new rmps.importer.ImportDefinitionEditor._PropertyRow({
			advancedSection: this,
			editor: this.editor,
			_propertyDescription: messages.transferTimeOutText,
			_value: {
				str : this._definition.transferTimeOut,
				advanced : this,
				_updateFunc : function () {
					this.advanced._definition.transferTimeOut = this.str;
				},
			},
			_originalValue: this._originalDefinition.transferTimeOut,
			_tooltipText: messages.transferTimeOutTooltip,
			_default: TRANSFER_TIMEOUT_DEFAULT,
			
		});
		
		var antExecutionTimeOutRow = new rmps.importer.ImportDefinitionEditor._PropertyRow({
			advancedSection: this,
			editor: this.editor,
			_propertyDescription: messages.antExecutionTimeOutText,
			_value: {
				str : this._definition.antExecutionTimeOut,
				advanced : this,
				_updateFunc : function() {	
					this.advanced._definition.antExecutionTimeOut = this.str;
				},
			},
			_originalValue: this._originalDefinition.antExecutionTimeOut,
			_tooltipText: messages.antExecutionTimeOutTooltip,
			_default: ANT_EXECUTION_TIMEOUT_DEFAULT,
		});
		
		this._table.tbody.insertBefore(antExecutionTimeOutRow.domNode, this._table.tbody.firstChild);
		this._table.tbody.insertBefore(transferTimeOutRow.domNode, this._table.tbody.firstChild);
		
		transferTimeOutRow.startup();
		antExecutionTimeOutRow.startup();
	},
});

dojo.declare("rmps.importer.ImportDefinitionEditor._PropertyRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<span dojoAttachPoint="_property">' +
				'</span>' +				
			'</td>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_currentValuePreview"></span>' +
					'<input class="editable" style="width: 100%;" dojoAttachPoint="_currentValue" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td class="defaultValue">' +
				'<span  dojoAttachPoint="_defaultValue"></span>' +
			'</td>' +
		'</tr>',
	
	advancedSection: null,
	editor: null,
	
	_propertyDescription: null,
	_value: null,
	_originalValue: null,	
	_tooltip: null,
	_default: null, 
	_tooltipText: null,
	
	postCreate: function() {
		
		this._property.appendChild(dojo.doc.createTextNode(this._propertyDescription));
		this._defaultValue.appendChild(dojo.doc.createTextNode(this._default));
		
		
		this._currentValuePreview.appendChild(dojo.doc.createTextNode(this._originalValue));
		this._currentValue.value = this._originalValue;
		
		if (!this.editor.isReadOnly) {
			connectOnInput(this._currentValue, this, function() {
				this._updateCurrentValue();
			});
		}		
		if (dojo.isIE) {
			dojo.toggleClass(this._property, "TooltipCursor", true);
			this.connect(this._property, "onmouseover", function(){
				dojo.toggleClass(this._property, "TooltipColor", true);
			});
			this.connect(this._property, "onmouseout", function(){
				dojo.toggleClass(this._property, "TooltipColor", false);
			});
		} else {
			dojo.toggleClass(this._property, "Tooltip", true);
		}
		// The tooltip sometimes has problems attaching to the object if without a small delay
		setTimeout(dojo.hitch(this, function() {
 			this._tooltip = new jazz.ui.HoverPopup({
				around: this._property,
				html: this._tooltipText,
				openDelay: 300,
				maxWidth: 800
			})
 		}), 50); 
 	},
	
	destroy: function() {
		if (this._tooltip) {
			this._tooltip.destroyRecursive();
			delete this._tooltip;
		}
	},
	
	focus: function() {
		this._currentValue.focus();
	},
	
	_updateCurrentValue: function() {
		this._value.str = dojo.trim(this._currentValue.value);
		this._value._updateFunc();
		this.editor._updateDirty(this._originalValue ? this._originalValue : "", this._value.str, this._currentValue);
		this._currentValuePreview.innerHTML = "";
		this._currentValuePreview.appendChild(dojo.doc.createTextNode(this._value.str));		
	},		
});

})();
