/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.ImportStatusEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("dijit._Widget");

dojo.require("jazz.util.date");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.HoverPopup");

dojo.require("jazz.ui.ResourceLink");

dojo.require("rmps.importer.Status");
dojo.require("rmps.importer.ImportClient");

dojo.require("rmps.core.error");

dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer", "ImportStatusEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.importer", "ImportStatusEditorMessages");

var ImportClient = rmps.importer.ImportClient;

dojo.declare("rmps.importer.ImportStatusEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "rmps-importer-status",
	
	// init properties
	isReadOnly: true,
	
	// private properties
	_table: null,
	_deferred: null,
	_statusRows: null, //[]
	_doNotDelete: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this.refresh();
	},
	
	constructor: function() {
		this.uaKey = "com.ibm.xtools.rmps.web.importer.page_ImportStatusEditor001";
	},
	
	refresh: function() {
		if (this._deferred) {
			return;
		}
		
		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		this.showLoading();
		
		this._deferred = ImportClient.getStatusForProject(this.getEditorInput().projectId);
		this._deferred.addBoth(this, function() {
			delete this._deferred;
			this.hideMessage();
		});
		
		this._deferred.addCallback(this, function(statuses) {
			
			function statusComparator(a, b) {
				return a.requestTime < b.requestTime ? 1 : a.requestTime == b.requestTime ? 0 : -1;
			}
			
			var table = new jazz.ui.StyledTable({
				columns: [
					{
						label: "",
						className: "state-icon"
					},
					{
						label: messages.state
					},
					{
						label: messages.requester
					},
					{
						label: messages.requestTime
					},
					{
						label: messages.startTime
					},
					{
						label: messages.duration
					},
					{
						label: messages.stage
					},
					{
						label: messages.actions,
						className: "actions"
					}
				]	
			});
			table.containerNode = table.tbody;
			this.containerNode.appendChild(table.domNode);
			
			statuses.sort(statusComparator);
			
			// these properties are updated by the rows
			this._statusRows = [];
			delete this._doNotDelete
			
			for (var i = 0, s; s = statuses[i]; ++i) {
				var row = new rmps.importer.ImportStatus._Row({editor: this, status: s});
				table.tbody.appendChild(row.domNode);
				row.startup();
			}
		});
		
		this._deferred.addErrback(this, function(error) {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_loadingStatus,
					closable: true
				});
		});
	},
	
	_canDelete: function(status) {
		for (var i = 0, s; s = this._statusRows[i]; ++i) {
			if (s.status.state == rmps.importer.StatusStates.ERROR ||
				(s.status.state == rmps.importer.StatusStates.CANCELED && !s.status.startTime) ||
				s.status.state == rmps.importer.StatusStates.COMPLETED ||
				s.status.state == rmps.importer.StatusStates.WARNING) {
				return s.status !== status;
			}
		}
		return true;
	}
});

dojo.declare("rmps.importer.ImportStatus._Row", dijit._Widget, {
	
	status: null,
	editor: null,
	
	_requesterDeferred: null,
	
	_duration: null,
	_validationDuration: null,
	
	_stateLink: null,
	_stateImg: null,
	_startTimeTd: null,
	_durationTd: null,
	_stageTd: null,
	
	_updateDeferred: null,
	_cancelImportDeferred: null,
	_deleteStatusDeferred: null,
	
	_popupContent: null,
	_popup: null,
	_actionBar: null,
	
	buildRendering: function() {
		this.domNode = this.containerNode = dojo.create("tr");
	},
	
	postCreate: function() {
		this.editor._statusRows.push(this);
		
		this._requesterDeferred = this.status.getRequester();
		
		// state img
		var td = dojo.create("td", null, this.domNode);
		this._stateImg = dojo.create("img", null, td);
		
		// state
		td = dojo.create("td", null, this.domNode);
		this._stateLink = dojo.create("a", { style: { cursor: "pointer" }, href: "#" }, td);
		
		this._updateState();
		
		this._popup = new jazz.ui.HoverPopup({
			around: this._stateLink,
			programmatic: false,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});
		dojo.connect(this._stateLink, "onkeydown", this, function(e) {
			if(!e|| e.keyCode== dojo.keys.ESCAPE){
				this._popup.close();
				return;
			}
			if(dojo.keys.ENTER == e.keyCode|| dojo.keys.SPACE == e.keyCode){
				setTimeout(dojo.hitch(this, function() {
					this._popup.open();
				}), 0);			
			}
			if(e.keyCode!= dojo.keys.TAB){
				dojo.stopEvent(e);
			}
		});
		
		dojo.connect(this._stateLink, "onclick", this, function(e) {
			dojo.stopEvent(e);
		});
		
		
		// requester
		td = dojo.create("td", null, this.domNode);
		var requesterLink = new jazz.ui.ResourceLink({
				uri: this.status.requester,
				label: "-"
			});
		td.appendChild(requesterLink.domNode);
		
		// request time
		td = dojo.create("td", null, this.domNode);
		td.appendChild(dojo.doc.createTextNode(
				dojo.date.locale.format(this.status.requestTime, {formatLength: "medium"})
			));
		
		// start time
		this._startTimeTd = dojo.create("td", null, this.domNode);
		this._updateStartTime();
		
		// duration
		this._durationTd = dojo.create("td", null, this.domNode);
		this._updateDuration();
		
		// stage
		this._stageTd = dojo.create("td", null, this.domNode);
		this._updateStage();
		
		// actions
		var actionsTd = dojo.create("td", null, this.domNode);
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionsTd.appendChild(this._actionBar.domNode);
		this._actionBar.startup();
		this._updateActions();
		
		// start update 
		if (this.status.state == rmps.importer.StatusStates.PENDING ||
				this.status.state == rmps.importer.StatusStates.IN_PROGRESS ||
				this.status.state == rmps.importer.StatusStates.CANCELING) {
			this._updateTimer = setInterval(dojo.hitch(this, this._update), 10000); // 10 seconds
		}
	},
	
	destroy: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
			delete this._popupContent;
		}
		if (this._popup) {
			this._popup.destroyRecursive();
			delete this._popup;
		}
		if (this._updateTimer) {
			clearInterval(this._updateTimer);
			delete this._updateTimer;
		}
		if (this._updateDeferred) {
			this._updateDeferred.cancel();
			delete this._updateDeferred;
		}
		if (this._deleteStatusDeferred) {
			this._deleteStatusDeferred.cancel();
			delete this._deleteStatusDeferred;
		}
		if (this._cancelImportDeferred) {
			this._cancelImportDeferred.cancel();
			delete this._cancelImportDeferred;
		}
		this.inherited(arguments);
	},
	
	_cancelImport: function() {
		this._actionBar.disableButton(this._cancelAction);
		
		this._cancelImportDeferred = ImportClient.cancelImport(this.status.uri);
		this._cancelImportDeferred.addBoth(this, function() {
			delete this._cancelImportDeferred;
			this._update();
		});
	},
	
	_deleteStatus: function() {
		this._actionBar.disableButton(this._deleteAction);
		
		this._deleteStatusDeferred = ImportClient.deleteStatus(this.status.uri);
		this._deleteStatusDeferred.addCallback(this, function() {
			delete this._deleteStatusDeferred;
			this.destroyRecursive();
		});
		this._deleteStatusDeferred.addErrback(this, function() {
			delete this._deleteStatusDeferred;
			this._update();
		});
	},
	
	_update: function() {
		if (this._updateDeferred) {
			return;
		}
		this._updateDeferred = ImportClient.getStatus(this.status.uri);
		this._updateDeferred.addBoth(this, function(status) {
			delete this._updateDeferred;
			if (status instanceof Error) {
				if (status.status == 410) {
					this.destroyRecursive();
				}
				return;
			}
			
			if (!status.uri) {
				// protect against corrupt data
				return;
			}
			
			var oldStatus = this.status;
			this.status = status;
			
			// check if the update interval can be cleared
			if (this.status.state != rmps.importer.StatusStates.PENDING &&
					this.status.state != rmps.importer.StatusStates.IN_PROGRESS &&
					this.status.state != rmps.importer.StatusStates.CANCELING) {
				clearInterval(this._updateTimer);
				delete this._updateTimer;
			}
			
			if (this.status.importer_stage != oldStatus.importer_stage) {
				this._updateStage();
			}
			
			if (this.status.state != oldStatus.state) {
				this._updateActions();
				this._updateState();
				if (oldStatus.state == rmps.importer.StatusStates.PENDING) {
					this._updateStartTime();
				}
			}
			if (this.status.state != rmps.importer.StatusStates.PENDING) {
				this._updateDuration();
			}
		});
	},
	
	_updateActions: function() {
		if (!this._deleteAction && (this.status.state == rmps.importer.StatusStates.ERROR ||
				this.status.state == rmps.importer.StatusStates.DROPPED ||
				this.status.state == rmps.importer.StatusStates.CANCELED ||
				this.status.state == rmps.importer.StatusStates.COMPLETED ||
				this.status.state == rmps.importer.StatusStates.WARNING)) {
			if (this._cancelAction) {
				this._actionBar.removeItem(this._cancelAction);
				delete this._cancelAction;
			}
			
			// do not allow the first status affecting resource state to be deleted
			if (this.editor._canDelete(this.status)) {
				this._deleteAction = this._actionBar.addButton(this, this._deleteStatus, {
					label: messages.deleteStatus,
					imgSrc: dojo.moduleUrl("rmps.importer", "graphics/icons/remove.gif")
				});
			} else {
				if (this.editor._doNotDelete && this.editor._doNotDelete !== this) {
					var dnd = this.editor._doNotDelete;
					delete this.editor._doNotDelete;
					dnd._updateActions();
				}
				this.editor._doNotDelete = this;
			}
			
		} else if (!this._cancelAction) {
			if (this._deleteAction) {
				this._actionBar.removeItem(this._deleteAction);
				delete this._deleteAction;
			}
			
			this._cancelAction = this._actionBar.addButton(this, this._cancelImport, {
					label: messages.cancelImport,
					imgSrc: dojo.moduleUrl("rmps.importer", "graphics/icons/cancel.gif")
				});
			
			if (this.status.state == rmps.importer.StatusStates.CANCELING) {
				this._actionBar.disableButton(this._cancelAction);
			}
		}
	},
	
	_updateStage: function() {
		this._stageTd.innerHTML = "";
		this._stageTd.appendChild(dojo.doc.createTextNode(this.status.importer_stage ? this.status.importer_stage.name : "-"));
	},
	
	_updateState: function() {
		this._stateImg.src = this.status.state.icon;
		this._stateLink.innerHTML = "";
		this._stateLink.appendChild(dojo.doc.createTextNode(this.status.state.name));
	},
	
	_updateStartTime: function() {
		this._startTimeTd.innerHTML = "";
		if (this.status.startTime) {
			this._startTimeTd.appendChild(dojo.doc.createTextNode(
				dojo.date.locale.format(this.status.startTime, {formatLength: "medium"})
			));
		} else {
			this._startTimeTd.appendChild(dojo.doc.createTextNode("-"));
		}
	},
	
	_updateDuration: function() {
		this._durationTd.innerHTML = "";
		
		var timestamp = new Date();
		var endTime = this.status.stopTime || this.status.currentTime;
		
		if (this.status.startTime && endTime) {
			this._duration = jazz.util.date.formatDuration(endTime.getTime() - this.status.startTime.getTime());
			this._durationTd.appendChild(dojo.doc.createTextNode(this._duration));
		} else {
			this._durationTd.appendChild(dojo.doc.createTextNode("-"));
		}
		
		endTime = this.status.validation_stopTime || this.status.currentTime;
		if (this.status.validation_startTime && endTime) {
			this._validationDuration = jazz.util.date.formatDuration(endTime.getTime() - this.status.validation_startTime.getTime());
		} else {
			this._validationDuration = "-";
		}
	},
	
	_createPopupContents: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
		this._popupContent = new rmps.importer.ImportStatus._PopupContent({row: this});
		this._popup.content = this._popupContent.domNode;
		
		if (this.status.state != rmps.importer.StatusStates.PENDING &&
				this.status.state != rmps.importer.StatusStates.IN_PROGRESS &&
				this.status.state != rmps.importer.StatusStates.CANCELING) {
			delete this._popup.onOpen;
		}
	}
});

dojo.declare("rmps.importer.ImportStatus._PopupContent", dijit._Widget, {
	
	"class": "rmps-importer-status-popup",
	
	row: null,
	
	postCreate: function() {
		this._createValidationSection();
		this._createImporterSection();
		this._createErrorSection();
	},
	
	_createNode: function(tag, refNode, value) {
		var node = dojo.create(tag, null, refNode);
		if (dojo.isFunction(value)) {
			value(node);
		} else {
			node.appendChild(dojo.doc.createTextNode(value));
		}
		return node;
	},
	
	_createCanceledSection: function() {
		var status = this.row.status;
		
		if (!status.canceller) {
			return;
		}
		
		var table = dojo.create("table", null, this.domNode);
		var tbody = dojo.create("tbody", null, table);
		
		tr = dojo.create("tr", null, tbody);
		this._createNode("td", tr, messages.label_canceller);
		this._createNode("td", tr, dojo.hitch(this, function(parentNode) {
			status.getCanceller().addCallback(this, function(user) {
				parentNode.appendChild(dojo.doc.createTextNode(user.name));
			});
		}));
	},
	
	_createGeneralSection: function() {
		var status = this.row.status;
		
		var table = dojo.create("table", null, this.domNode);
		var tbody = dojo.create("tbody", null, table);
		
		var tr = dojo.create("tr", null, tbody);
		
		this._createNode("th", tr, messages.state);
		this._createNode("th", tr, messages.requester);
		this._createNode("th", tr, messages.requestTime);
		this._createNode("th", tr, messages.startTime);
		this._createNode("th", tr, messages.duration);
		
		if (status.importer_stage) {
			this._createNode("th", tr, messages.stage);
		}
		
		tr = dojo.create("tr", null, tbody);
		
		this._createNode("td", tr, status.state.name);
		this._createNode("td", tr, dojo.hitch(this, function(parentNode) {
			this.row._requesterDeferred.addCallback(this, function(user) {
				parentNode.appendChild(dojo.doc.createTextNode(user.name));
			});
		}));
		this._createNode("td", tr, status.requestTime ? dojo.date.locale.format(status.requestTime, {formatLength: "medium"}) : "-");
		this._createNode("td", tr, status.startTime ? dojo.date.locale.format(status.startTime, {formatLength: "medium"}) : "-");
		this._createNode("td", tr, this.row._duration ? this.row._duration : "-");
		
		if (status.importer_stage) {
			this._createNode("td", tr, status.importer_stage.name);
		}
	},
	
	_createValidationSection: function() {
		var status = this.row.status;
		
		if (!status.validation_startTime) {
			return;
		}
		
		var table = dojo.create("table", null, this.domNode);
		var tbody = dojo.create("tbody", null, table);
		
		var tr = dojo.create("tr", null, tbody);
		this._createNode("th", tr, messages.validation);
		
		tr = dojo.create("tr", null, tbody);
		this._createNode("td", tr, messages.label_startTime);
		this._createNode("td", tr, dojo.date.locale.format(status.validation_startTime, {formatLength: "medium"}));
		this._createNode("td", tr, messages.label_duration);
		this._createNode("td", tr, this.row._validationDuration ? this.row._validationDuration : "-");
	},
	
	_createErrorSection: function() {
		var status = this.row.status;
		
		if (!(status.exception || status.dropCause)) {
			return;
		}
		
		var table = dojo.create("table", null, this.domNode)
		var tbody = dojo.create("tbody", null, table);
		
		var tr = dojo.create("tr", null, tbody);
		this._createNode("th", tr, messages.errors);
		
		tr = dojo.create("tr", null, tbody);
		 
		if (status.dropCause) {
			var tr = dojo.create("tr", null, tbody);
			this._createNode("td", tr, status.dropCause.name);
		} else if (status.importer_permaLinkConflicts) {
			this._createListRow(tbody, "permaLinkConflicts", messages.label_permaLinkConflicts, status.importer_permaLinkConflicts);
		} else if (status.importer_importEngineErrors) {
			var tr = dojo.create("tr", null, tbody);
			this._createNode("td", tr, messages.label_importEngineErrors);
			this._createNode("td", tr,  status.importer_importEngineErrors);
		} else if (status.exception) {
			var tr = dojo.create("tr", null, tbody);
			this._createNode("td", tr, messages.label_exception);
			this._createNode("td", tr, status.exception);
		}
		
	},
	
	_createImporterSection: function() {
		var status = this.row.status;
		
		if (status.state == rmps.importer.StatusStates.PENDING ||
			status.state == rmps.importer.StatusStates.DROPPED) {
			return;
		}
		
		var table = dojo.create("table", null, this.domNode);
		var tbody = dojo.create("tbody", null, table);
		
		var tr = dojo.create("tr", null, tbody);
		this._createNode("th", tr, messages.details);
		
		tr = dojo.create("tr", null, tbody);
		
		this._createNode("td", tr, messages.label_numberOfResources);
		this._createNode("td", tr, status.importer_numberOfResources);
		
		tr = dojo.create("tr", null, tbody);
		this._createNode("td", tr, messages.label_numberOfElements);
		this._createNode("td", tr, status.importer_numberOfElements);
		
		tr = dojo.create("tr", null, tbody);
		this._createNode("td", tr, messages.label_numberOfResourcesDeleted);
		this._createNode("td", tr, status.importer_numberOfResourcesDeleted);		
				
		//status of baseline, if one was requested
		if (status.importer_baselineRequestStatus) {
			tr = dojo.create("tr", null, tbody);
			this._createNode("td", tr, "Snapshot:");
			this._createNode("td", tr, status.importer_baselineRequestStatus.name);			
		}
		
		//warnings
		
		table = dojo.create("table");
		tbody = dojo.create("tbody", null, table);
		
		if (status.importer_unsetPathMapVariables) {
			this._createListRow(tbody, "unsetPathMapVariables", messages.label_unsetPathMapVariables, status.importer_unsetPathMapVariables);
		}
		
		if (status.importer_invalidProjectPaths) {
			this._createListRow(tbody, "invalidProjectPaths", messages.label_invalidProjectPaths, status.importer_invalidProjectPaths);
		}
		
		if (status.importer_errorResources) {
			this._createListRow(tbody, "errorResources", messages.label_errorResources, status.importer_errorResources);
		}
		
		if (status.importer_ignoredPaths) {
			this._createListRow(tbody, "ignoredPaths", messages.label_ignoredPaths, status.importer_ignoredPaths);
		}
		
		if (status.importer_danglingReferences) {
			this._createListRow(tbody, "danglingReferences", messages.label_danglingReferences, status.importer_danglingReferences);
		}
		
		if (tbody.firstChild) {
			this.domNode.appendChild(table);
		}
	},
	
	_createListRow: function(tbody, id, label, values) {
		var tr = dojo.create("tr", null, tbody);
		this._createNode("td", tr, label);
		
		var valueTd = dojo.create("td", null, tr);
		var ul = dojo.create("ul", null, valueTd);
		for (var i = 0; i < values.length; ++i) {
			this._createNode("li", ul, values[i]);
		}
	}
});

})();
