/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.pages.ImportComponentMenu");

dojo.require("rmps.importer.ImportDefinitionEditorInput");
dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.importer.RequestImportEditorInput");

dojo.require("rmps.app.ui.EditorUtil");

dojo.require("jazz.app._ComponentMenu");
dojo.require("jazz.app.context");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer.pages", "ImportPageMessages");

var messages = dojo.i18n.getLocalization("rmps.importer.pages", "ImportPageMessages");

var PAGE_ID = "rmps.importer.import";

dojo.declare("rmps.importer.pages.ImportComponentMenu", jazz.app._ComponentMenu, {
	
	_importStatusTitle: null,
	_statuses: null, //[]
	
	postCreate: function() {
		var projectId = jazz.app.context.get().itemId;
		var defEditorInput = new rmps.importer.ImportDefinitionEditorInput({projectId: projectId});
		var riEditorInput = new rmps.importer.RequestImportEditorInput({projectId: projectId});
		var statusEditorInput = new rmps.importer.ImportStatusEditorInput({projectId: projectId});
		
		this.addTitle(messages.import_);
		this.addItemByArgs({
				editorInput: defEditorInput,
				label: defEditorInput.getName(),
				//icon: defEditorInput.getIconUri(),
				href: defEditorInput.getPresentationUri()
			});
		this.addItemByArgs({
				editorInput: riEditorInput,
				label: riEditorInput.getName(),
				//icon: riEditorInput.getIconUri(),
				href: riEditorInput.getPresentationUri()
			});
		this.addItemByArgs({
				editorInput: statusEditorInput,
				label: statusEditorInput.getName(),
				//icon: statusEditorInput.getIconUri(),
				href: statusEditorInput.getPresentationUri()
			});
		
		// global provider status
		var gps = dojo.getObject("rmps.importer.GlobalProviderStatus");
		if (gps) {
			this.connect(gps, "onChange", this._updateProviderStatus);
			this._updateProviderStatus(gps);
		}
	},
	
	getSelected: function() {
		var sel = [];
		if (net.jazz.ajax.ui.PlatformUI.getWorkbench().currentPageId == PAGE_ID) {
			var editorUtil = rmps.app.ui.EditorUtil.getInstance(PAGE_ID);
			var aei = editorUtil.getActiveEditorInput();
			var items = this.getItems();
			var href = location.href;
			for (var i = 0; i < items.length; i++) {
				var item = items[i];
				if (item && (item.editorInput && item.editorInput.getId() === aei.getId() || (item.href && href.indexOf(item.href) > 0))) {
					sel.push(item);
				}
			}
		}
		return sel;
	},
	
	_updateProviderStatus: function(gps) {
		if (this._statuses && this._statuses.length > 0) {
			dojo.forEach(this._statuses, function(item) {
					this.remove(item).destroyRecursive();
				}, this);
		}
		
		if (gps.providerStatuses.length === 0) {
			this._importStatusTitle && this.remove(this._importStatusTitle);
			delete this._importStatusTitle;
		} else if (!this._importStatusTitle) {
			this._importStatusTitle = this.addTitle(messages.importStatus, {wrapHint: 2}); // 2 = FORCE_WRAP
		}
		
		this._statuses = [];
		for (var i = 0, ps; ps = gps.providerStatuses[i]; ++i) {
			var uri = ps.importWebUri;
			
			// for development debugging
			if (dojo.config.isDebug) {
				if (uri.indexOf("?") > 0) {
					uri = uri.replace("?", "?debug=true&");
				} else if (uri.indexOf("#") > 0) {
					uri = uri.replace("#", "?debug=true#");
				} else {
					uri += "?debug=true";
				}
			}
			
			this._statuses.push(
				this.addItemByArgs({
						label: ps.providerName,
						icon: ps.importState.icon,
						title: ps.importState.name,
						href: uri
					}, this._importStatusTitle.getIndexInParent() + 1 + i));
		}
	},
	
	_refreshStatus: function() {
		var editorUtil = rmps.app.ui.EditorUtil.getInstance(PAGE_ID);
		var statusEditor = editorUtil.getEditor(
				new rmps.importer.ImportStatusEditorInput({projectId: jazz.app.context.get().itemId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	}
});
})();