/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.importer.pages.ImportPage");

dojo.require("dijit._Widget");

dojo.require("rmps.app.ui.EditorUtil");
dojo.require("rmps.app.ui.EditorView");

dojo.require("rmps.importer.pages.ImportPageController");

dojo.require("rmps.importer.ImportDefinitionEditorInput");
dojo.require("rmps.importer.ImportStatusEditorInput");
dojo.require("rmps.importer.RequestImportEditorInput");

dojo.require("jazz.ui.SideNavigationLayout");

dojo.require("net.jazz.ajax.ui.ErrorPage");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("rmps.core.error");

dojo.require("rmps.importer.util");

dojo.require("jazz.app.context");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.importer.pages", "ImportPageMessages");

var messages = dojo.i18n.getLocalization("rmps.importer.pages", "ImportPageMessages");

var Util = rmps.importer.util;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var PAGE_REMOVAL_ID = "rmps.importer.pages.ImportPage.remove";
var PAGE_ID = "rmps.importer";
var ACTION_PREFIX = "rmps.importer.";
var ACTION_NAMES = [
	"_default",
	"status",
	"viewDefinition",
	"request"
];

dojo.declare("rmps.importer.pages.ImportPage", dijit._Widget, {
	
	"class": "rmps-importer-import",
	
	// private properties
	_controller: null,
	_sideNav: null,
	
	_statusSection: null,
	_definitionSection: null,
	_highlightedNode: null,
	_importButton: null,
	
	postCreate: function() {
		this.containerNode = this.domNode;
		
		var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		
		if (dojo.getObject(PAGE_REMOVAL_ID)) {
			// register void actions
			for (var i = 0; i < ACTION_NAMES.length; i++) {
				actionRegistry.registerAction(ACTION_PREFIX + ACTION_NAMES[i], new Function());
			}
		
			var errorPage = new net.jazz.ajax.ui.ErrorPage({
					message: messages.pageRemoved,
					hideRunDefaultAction: true
				});
			this.domNode.appendChild(errorPage.domNode);
			return;
		}
		
		var projectId = jazz.app.context.get().itemId;
		this._importDefinitionEditorInput = new rmps.importer.ImportDefinitionEditorInput({projectId: projectId});
		this._importStatusEditorInput = new rmps.importer.ImportStatusEditorInput({projectId: projectId});
		this._requestImportEditorInput = new rmps.importer.RequestImportEditorInput({projectId: projectId});

		
		// create editor util instance
		this._editorUtil = rmps.app.ui.EditorUtil.getInstance(PAGE_ID, this.domNode);
		this._editorUtil.defaultUri = this._importDefinitionEditorInput.getPresentationUri();
		
		// create controller and register actions
		this._controller = new rmps.importer.pages.ImportPageController(this);
		
		// register actions
		for (var i = 0; i < ACTION_NAMES.length; i++) {
			actionRegistry.registerAction(ACTION_PREFIX + ACTION_NAMES[i], this._controller, ACTION_NAMES[i]);
		}
		
		// init side navigation
		this._sideNav = new jazz.ui.SideNavigationLayout();
		
		var section = this._sideNav.addNavigationSection(dojo.doc.createElement("div"), messages.import_, true);
		
		this._definitionSection = section.addSectionRow(
				this._importDefinitionEditorInput.getName(),
				this._importDefinitionEditorInput.getPresentationUri(),
				this._importDefinitionEditorInput.getIconUri(), null);
		
		this._requestImportSection = section.addSectionRow(
				this._requestImportEditorInput.getName(),
				this._requestImportEditorInput.getPresentationUri(),
				this._requestImportEditorInput.getIconUri(), null);
				
		this._statusSection = section.addSectionRow(
				this._importStatusEditorInput.getName(),
				this._importStatusEditorInput.getPresentationUri(),
				this._importStatusEditorInput.getIconUri(), null);
		
		// add status refresh command
		section.addArtifactCommand(this._statusSection,
			messages.refresh,
			dojo.hitch(this, function(e) {
				dojo.stopEvent(e);
				this._refreshStatus();
				window.location.href = this._importStatusEditorInput.getPresentationUri();
			}),
			"refresh", true);
			
		this.domNode.appendChild(this._sideNav.domNode);
		this._sideNav.startup();
		
		var editorView = new rmps.app.ui.EditorView();
		this._sideNav.setMainContent(editorView.domNode);
		editorView.startup();
	},
	
	destroy: function() {
		this._sideNav.destroyRecursive();
		this.inherited(arguments);
	},
	
	_refreshStatus: function() {
		var statusEditor = this._editorUtil.getEditor(this._importStatusEditorInput);
		if (statusEditor) {
			statusEditor.refresh();
		}
	},
	
	showStatus: function() {
		this._highlightNode(this._statusSection);
		this._editorUtil.openEditor(this._importStatusEditorInput);
	},
	
//	_getEditorView: function() {
//		if (!this._editorView) {
//			this._editorView = new rmps.app.ui.EditorView({
//				editorUtil: this._editorUtil
//			});
//			this._editorView.startup();
//			this._editorUtil.domNode = this._editorView.domNode;
//			this._editorUtil.defaultUri = this._importDefinitionEditorInput.getPresentationUri();
//		}
//		return this._editorView;
//	},
	
	showDefinition: function(args) {
		this._highlightNode(this._definitionSection);
		this._editorUtil.openEditor(this._importDefinitionEditorInput);
	},
	
	showRequestImport: function(args) {
		this._highlightNode(this._requestImportSection);
		this._editorUtil.openEditor(this._requestImportEditorInput);
	},
	
	_highlightNode: function(node){
		if (this._highlightedNode != node) {
			if (this._highlightedNode) {
				dojo.removeClass(this._highlightedNode, "selected");
			}
			dojo.addClass(node, "selected");
			this._highlightedNode = node;
		}
	}
});

})();
