/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
	dojo.provide("uml.rmps.pages.SamplePage");

	dojo.require("rmps.importer.ImportClient");
	dojo.require("rmps.ui.DetailedProgress");
	dojo.require("rmps.core.RootServices");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

	dojo.require("dojo.i18n");

	dojo.require("net.jazz.ajax.ui.PlatformUI");

	dojo.requireLocalization("uml.rmps.pages",
			"SamplePageMessages");

	var progress;
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var messages = dojo.i18n.getLocalization("uml.rmps.pages",
			"SamplePageMessages");

	var SAMPLE_KEY = "SAMPLE_KEY"; //$NON-NLS-1$
	var LINE_BREAK_TAG = "<BR><BR>"; //$NON-NLS-1$
	var INSTRUCTIONS = messages.instructions1
			+ LINE_BREAK_TAG
			+ dojo.string.substitute(messages.instructions2,
					[ net.jazz.ajax._dataServerRoot + "/admin" ]);
	var STEP1_DESCRIPTION = messages.step1Description1 + LINE_BREAK_TAG
			+ messages.step1Description2;

	dojo.declare("uml.rmps.pages.SamplePage", [
			dijit._Widget, dijit._Templated ], {		
		
		"class": "samplePage",

		templatePath : dojo.moduleUrl("uml.rmps", "pages/templates/SamplePage.html"),

		constructor : function() {
			this.messages = messages = dojo.i18n.getLocalization(
					"uml.rmps.pages", "SamplePageMessages");
		},

		postCreate : function() {
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry
					.registerAction("uml.rmps.sample", new Function());

			this.progress = new rmps.ui.DetailedProgress({
				progressTitle : this.messages.viewProgress
			});

			this.progress.addCategoryDescriptor(SAMPLE_KEY, {
				buttonTitle : messages.createButton,
				instructions : INSTRUCTIONS,
				title : messages.createSampleTitle
			});

			this.progress.addStep(SAMPLE_KEY, {
				header : messages.step1Header,
				description : STEP1_DESCRIPTION,
				performCallback : dojo.hitch(this, this.createSampleProject)
			});

			this.progress.render();

			this.mainNode.appendChild(this.progress.domNode);
		},

		createSampleProject : function(infoArea, detailedProgress) {
			
			var deferred = rmps.importer.ImportClient.importSample();
			deferred.addCallback(this, function(projectAreaLocation) {

				var segments = projectAreaLocation.split('/');
				var projectIdToMatch = segments.length ? segments[segments.length - 1] : null;
				var args = {
					url: rmps.app.core.util.getProxyRequestUri(rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas")),
					handleAs: "text",
					load: function(response, ioArgs) {
						var projectName, projectAreaURL;
						var doc = dojox.xml.DomParser.parse(response);
						var projects = doc.getElementsByTagName("jp06:project-area");
						for (var i = 0, p; p = projects[i]; ++i) {
							var uri = p.getElementsByTagName("jp06:url")[0].childNodes[0].nodeValue;
							var segments = uri.split('/');
							if (projectIdToMatch == segments[segments.length - 1]) {
								projectName = p.getAttribute("jp06:name");
								projectAreaURL = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "web") 
									+ "/projects/" + projectName;
								break;
							}								
						}				
						
						var msg = dojo.string.substitute(messages.success, [projectName])
								+ LINE_BREAK_TAG
								+ dojo.string.substitute(messages.statusLink,
										[ projectAreaURL + "#action=rmps.importer.status" ])
								+ LINE_BREAK_TAG
								+ dojo.string.substitute(messages.projectLink,
										[ projectAreaURL + "#action=rmps.app.welcome" ]);

						detailedProgress.notifySuccess(infoArea, msg);
					}
				};
				jazz.client.xhrGet(args);

			});
			deferred.addErrback(this, function(response) {
				if (response instanceof Error) {
					if (response.status && (response.status === 409)) {
						detailedProgress.notifyWarning(infoArea, messages.projectAlreadyExists);
						return;
					} else if (response.status && (response.status === 403)) {
						detailedProgress.notifyError(infoArea, messages.unauthorizedUser);
						return;
					}
				}
				detailedProgress.notifyError(infoArea, messages.unknownError + LINE_BREAK_TAG + response);
			});
		}
	});

})();
